# -*- coding: utf-8 -*-

"""package scinstr
author    Benoit Dubois
copyright FEMTO ENGINEERING, 2026
license   LGPL v3.0+
brief     Base class for handling NF LI56[45|50] devices
"""

import time
from collections import OrderedDict
from scinstr.lockin.li5645_services import HwIntfService

SENSITIVITY = OrderedDict(
    [
        ("1 V", 0),
        ("500 mV", 1),
        ("200 mV", 2),
        ("100 mV", 3),
        ("50 mV", 4),
        ("20 mV", 5),
        ("10 mV", 6),
        ("5 mV", 7),
        ("2 mV", 8),
        ("1 mV", 9),
        ("500 uV", 10),
        ("200 uV", 11),
        ("100 uV", 12),
        ("50 uV", 13),
        ("20 uV", 14),
        ("10 uV", 15),
        ("5 uV", 16),
        ("2 uV", 17),
        ("1 uV", 18),
        ("500 nV", 19),
        ("200 nV", 20),
        ("100 nV", 21),
        ("50 nV", 22),
        ("20 nV", 23),
        ("10 nV", 24),
    ]
)

# TCONSTANTS = OrderedDict(
#     [
#         ("10 us", 0),
#         ("30 us", 1),
#         ("100 us", 2),
#         ("300 us", 3),
#         ("1 ms", 4),
#         ("3 ms", 5),
#         ("10 ms", 6),
#         ("30 ms", 7),
#         ("100 ms", 8),
#         ("300 ms", 9),
#         ("1 s", 10),
#         ("3 s", 11),
#         ("10 s", 12),
#         ("30 s", 13),
#         ("100 s", 14),
#         ("300 s", 15),
#         ("1 ks", 16),
#         ("3 ks", 17),
#         ("10 ks", 18),
#         ("30 ks", 19),
#     ]
# )

# SAMPLE_RATES = OrderedDict(
#     [
#         ("62.5 mHz", 0),
#         ("125 mHz", 1),
#         ("250 mHz", 2),
#         ("500 mHz", 3),
#         ("1 Hz", 4),
#         ("2 Hz", 5),
#         ("4 Hz", 6),
#         ("8 Hz", 7),
#         ("16 Hz", 8),
#         ("32 Hz", 9),
#         ("64 Hz", 10),
#         ("128 Hz", 11),
#         ("256 Hz", 12),
#         ("512 Hz", 13),
#         ("trigger", 14),
#     ]
# )

# REFERENCE_TRIGGER = OrderedDict(
#     [("zero_crossing", 0), ("rising_edge", 1), ("faling_edge", 2)]
# )

# INPUT_SHIELD = OrderedDict([("float", 0), ("ground", 1)])

# INPUT_COUPLING = OrderedDict([("AC", 0), ("DC", 1)])

# INPUT_FILTER = OrderedDict(
#     [((False, False), 0), ((True, False), 1), ((False, True), 2), ((True, True), 3)]
# )

# RESERVE_MODE = OrderedDict([("high", 0), ("normal", 1), ("low", 2)])

# FILTER_SLOPE = OrderedDict(
#     [("6 dB/oct", 0), ("12 dB/oct", 1), ("18 dB/oct", 2), ("24 dB/oct", 3)]
# )

# FRONT_OUTPUT = OrderedDict([("display", 0), ("X", 1)])

# AUTO_OFFSET_ASYNC_CH = OrderedDict([("x", 1), ("y", 2), ("r", 3)])

# FAST_MODE = OrderedDict([("Off", 0), ("On (Dos)", 1), ("On (Windows)", 2)])


# =============================================================================
class Li5645(object):
    """Base class for NF LI56[45|50] Lock-In Amplifier devices."""

    def __init__(self, hw_intf: HwIntfService) -> None:
        self.hw_intf = hw_intf

    def connect(self) -> bool:
        return self.hw_intf.connect()

    def close(self) -> None:
        return self.hw_intf.close()

    def write(self, msg: str) -> int:
        return self.hw_intf.write(msg)

    def read(self, length: int) -> str:
        return self.hw_intf.read(length)

    def query(self, msg: str) -> str:
        return self.hw_intf.query(msg)

    @property
    def idn(self):
        """Instrument identification."""
        return self.query("*IDN?")

    def reset(self):
        """Reset GPIB interface of Stanford device"""
        self.write("*RST")

    def status_byte(self, bit=None):
        """Serial poll status byte, a number between 0-255.
        User can request a specific bit value.
        """
        if bit is None:
            return int(self.query("*STB?"))
        else:
            return int(self.query("*STB? {:d}".format(bit)))

    def wait_bit0(self):
        """Check the scan in progress bit in the Serial Poll Status
        Byte (bit 0) to determine when a scan is finished.
        """
        bit0 = self.status_byte(0)
        while bit0 == 0:
            time.sleep(1.0)
            bit0 = self.status_byte(0)

    def wait_bit1(self):
        """Check the command execution in progress bit in the Serial Poll
        Status Byte (bit 1) to determine when the function is finished.
        """
        bit1 = self.status_byte(1)
        while bit1 == 0:
            time.sleep(0.5)
            bit1 = self.status_byte(1)

    @property
    def reference_phase_shift(self):
        """Getter of the phase shift of the reference."""
        return self.query("PHAS?")

    @reference_phase_shift.setter
    def reference_phase_shift(self, value):
        """Setter of the phase shift of the reference."""
        self.write("PHAS{:.2f}".format(value))

    @property
    def reference_frequency(self, outn: int = 1) -> str | None:
        """Getter of the reference source.
        :param outn: source output (int, 1 or 2)
        """
        if outn not in (1, 2):
            return
        return self.query(f":SOUR:FREQ{outn:d}?")

    @reference_frequency.setter
    def reference_frequency(self, value: float, outn: int = 1):
        """Setter of the reference source."""
        if outn not in (1, 2):
            return
        self.write(f":SOUR:FREQ{outn:d} {value:.5E}")

    @property
    def reference_amplitude(self) -> str:
        """Getter of the amplitude of the sine:SOURce:VOLTag output."""
        return self.query("SOUR:VOLT:AMPL?")

    @reference_amplitude.setter
    def reference_amplitude(self, value: float) -> None:
        """Setter of the amplitude of the sine output."""
        self.write(f"SOUR:VOLT:AMPL {value:.3E}")

    # --------------------------------
    # GAIN and TIME CONSTANT COMMANDS.
    # --------------------------------

    @property
    def sensitivity(self):
        """Getter of the sensitivity tuning."""
        return self.query("VOLT:AC:RANG?")

    @sensitivity.setter
    def sensitivity(self, value):
        """Setter of the sensitivity tuning."""
        self.write(f"VOLT:AC:RANG {value}")

    @property
    def time_constants(self):
        """Get time constant."""
        return self.query("FILT:TCON?")

    @time_constants.setter
    def time_constants(self, value):
        """Set time constant."""
        self.write(f"FILT:TCON {value}")

    @property
    def filter_db_per_oct(self):
        """Get the low pass filter slope."""
        return self.query("FILT:SLOP?")

    @filter_db_per_oct.setter
    def filter_db_per_oct(self, value):
        """Set the low pass filter slope."""
        self.write(f"FILT:SLOP {value:d}")

    @property
    def sync_filter(self):
        """Get synchronous filter status."""
        return self.query("FILT:TYPE?")

    @sync_filter.setter
    def sync_filter(self, value):
        """Set synchronous filter status."""
        self.write(f"FILT:TYPE {value}")

    # SETUP COMMANDS

    # @property
    # def remote(self):
    #     """Lock Front panel."""
    #     self.write("OVRM?")

    # @remote.setter
    # def remote(self, value):
    #     """Lock Front panel."""
    #     self.write("OVRM {:d}".format(value))

    # @property
    # def key_click_enabled(self):
    #     """Get key click status."""
    #     return self.query("KCLK?")

    # @key_click_enabled.setter
    # def key_click_enabled(self, value):
    #     """Set key click status."""
    #     self.write("KCLK {}".format(value))

    # @property
    # def alarm_enabled(self):
    #     """Get alarm status."""
    #     return self.query("ALRM?")

    # @alarm_enabled.setter
    # def alarm_enabled(self, value):
    #     """Set alarm status."""
    #     self.write("ALRM {}".format(value))

    # def recall_state(self, location):
    #     """Recalls instrument state in specified non-volatile location.
    #     :param location: non-volatile storage location.
    #     """
    #     self.write("RSET {}".format(location))

    # def save_state(self, location):
    #     """Saves instrument state in specified non-volatile location.
    #     Previously stored state in location is overwritten (no error
    #     is generated).
    #     :param location: non-volatile storage location.
    #     """
    #     self.write("SSET {}".format(location))

    # ---------------
    # AUTO FUNCTIONS.
    # ---------------

    # def auto_gain_async(self):
    #     """Equivalent to press the Auto Gain key in the front panel.
    #     Might take some time if the time constant is long.
    #     Does nothing if the constant is greater than 1 second.
    #     """
    #     self.write("AGAN")

    # def auto_gain(self):
    #     """Synced auto gain command."""
    #     self.auto_gain_async()
    #     self.wait_bit1()

    # def auto_reserve_async(self):
    #     """Equivalent to press the Auto Reserve key in the front panel.
    #     Might take some time if the time constant is long.
    #     """
    #     self.write("ARSV")

    # def auto_reserve(self):
    #     """Synced auto reserve command"""
    #     self.auto_reserve_async()
    #     self.wait_bit1()

    # def auto_phase_async(self):
    #     """Equivalent to press the Auto Phase key in the front panel.
    #     Might take some time if the time constant is long.
    #     Does nothing if the phase is unstable.
    #     """
    #     self.write("ARSV")

    # def auto_phase(self):
    #     """Synced auto phase command."""
    #     self.auto_phase_async()
    #     self.wait_bit1()

    # def auto_offset_async(self, channel):
    #     """Automatically offset a given channel to zero.
    #     Is equivalent to press the Auto Offset Key in the front panel.
    #     :param channel: the name of the channel.
    #     :returns: None
    #     """
    #     self.write("AOFF {}".format(channel))

    # ----------------------
    # DATA STORAGE COMMANDS.
    # ----------------------

    # @property
    # def sample_rate(self):
    #     """Queries the data sample rate."""
    #     return self.query("SRAT?")

    # @sample_rate.setter
    # def sample_rate(self, value):
    #     """Sets the data sample rate."""
    #     self.write("SRAT {}".format(value))

    # @property
    # def single_shot(self):
    #     """End of buffer mode getter:
    #     - if True (1 shot mode), at the end of the buffer, data storage stops
    #     and an audio alarm sounds.
    #     - if False (Loop mode), data storage continues continues at the end of
    #     the buffer.
    #     Note: If loop mode selected, make sure to pause data storage before
    #     reading the data to avoid confusion about which point is the most
    #     recent.
    #     """
    #     return self.query("SEND?")

    # @single_shot.setter
    # def single_shot(self, value):
    #     """End of buffer mode setter."""
    #     self.write("SEND {}".format(value))

    # def trigger(self):
    #     """Software trigger command. This command has the same effect as
    #     a trigger at the rear panel trigger input.
    #     """
    #     self.write("TRIG")

    # @property
    # def trigger_start_mode(self):
    #     """Getter of trigger start mode."""
    #     return self.query("TSTR?")

    # @trigger_start_mode.setter
    # def trigger_start_mode(self, value):
    #     """Setter of trigger start mode."""
    #     self.write("TSTR {}".format(value))

    # def start_data_storage(self):
    #     """Start or resume data storage. Command is ignored if
    #     storage is already in progress.
    #     """
    #     self.write("STRT")

    # def pause_data_storage(self):
    #     """Pause data storage.  If storage is already paused
    #     or reset then this command is ignored.
    #     """
    #     self.write("PAUS")

    # def reset_data_storage(self):
    #     """Reset data buffers. The command can be sent at any time, any storage
    #     in progress, paused or not, will be reset. The command will erase
    #     the data buffer.
    #     """
    #     self.write("REST")

    # ---------------
    # OAUX See above.
    # ---------------

    # def buffer_length(self):
    #     """Query the number of points stored in the buffer.
    #     :returns: the number of points stored in the buffer.
    #     """
    #     return int(self.query("SPTS?"))

    # @property
    # def fast_mode(self):
    #     """Query the data transfer mode value:
    #     - 0: Off
    #     - 1: On (DOS programs or other dedicated data collection computers)
    #     - 2: On (Windows Operating System Programs)
    #     """
    #     return self.query("FAST?")

    # @fast_mode.setter
    # def fast_mode(self, value):
    #     """Set the data transfer mode on and off."""
    #     self.write("FAST {:d}".format(value))

    # def start_fast_data_storage(self):
    #     """Start the datat storage after turning on fast data transfert."""
    #     self.write("STRD")

    # -----------------------
    # DATA TRANSFER COMMANDS.
    # -----------------------

    # def analog_value(self, key):
    #     """The command reads the value of X, Y, R, θ, CH1 or CH2 display.
    #     Available key = {'x', 'y', 'r', 't', '1', '2'}.
    #     """
    #     if key in "xyrt":
    #         return self.query("OUTP? {}".format(key))
    #     else:
    #         return self.query("OUTR? {}".format(key))

    def set_measure(self, measure: str, channel:int = 1) -> None:
        """From "REAL|MLIN|IMAG|PHAS|NOIS|AUX1|REAL2|MLIN2"
        """
        self.write(f"CALC{channel}:FORM {measure}")

    def measure(self, channel:int = 1) -> str:
        """The command return the value from X, Y, R, θ and input referred noise density.
        :returns: data value recorded (float)
        """
        return self.query(f"CALC{channel}:FORM?")

    # def read_buffer(self, start=0, length=None, _format="a"):
    #     """Queries points stored in the Channel buffer
    #     :param start: Index of the buffer to start.
    #     :param length: Number of points to read.
    #                    Defaults to the number of points in the buffer.
    #     :param _format: Transfer format
    #                   'a': ASCII (slow)
    #                   'b': IEEE Binary (fast) - NOT IMPLEMENTED
    #                   'c': Non-IEEE Binary (fastest) - NOT IMPLEMENTED
    #     """
    #     if _format == "c":
    #         cmd = "TRCL"
    #     elif _format == "b":
    #         cmd = "TRCB"
    #     else:
    #         cmd = "TRCA"
    #     if not length:
    #         length = self.buffer_length
    #     self.write("{}? {},{}".format(cmd, start, length))
    #     if cmd == "TRCA":
    #         data = self.read()
    #         return [float(x) for x in data.split(",")]
    #     elif cmd == "TRCB":
    #         return self.read()
    #     else:
    #         return self.read()

    # @property
    # def display(self, channel):
    #     """Query the display source.
    #     :param channel: 1 or 2 i.e. CH1 or CH2 (int)
    #     """
    #     return [int(var) for var in self.query("DDEF? {}".format(channel)).split(",")]

    # @display.setter
    # def display(self, channel, display, ratio):
    #     """Set the display source.
    #     :param channel: 1 or 2 i.e. CH1 or CH2 (int)
    #     :param display: 0, 1, 2, 3, 4 i.e. X, R, Xnoise,
    #                     AUxIn1 or AuxIn2 (int))
    #     :param ratio: 0, 1, 2 i.e. none, AuxIn1, AuxIn2 (int)
    #     """
    #     self.write("DDEF {},{},{}".format(channel, display, ratio))

    # @property
    # def front_output(self, channel):
    #     """Query the output source.
    #     :param channel: 1 or 2 i.e. CH1 or CH2 (int)
    #     """
    #     return self.query("FPOP? {}".format(channel))

    # @front_output.setter
    # def front_output(self, channel, output):
    #     """Set the output source.
    #     :param channel: 1 or 2 i.e. CH1 or CH2 (int)
    #     :param output: X, Y (output=1) or Display (output=0) (int)
    #     """
    #     self.write("FPOP {},{}".format(channel, output))



# -*- coding: utf-8 -*-

"""package scinstr
author    Benoit Dubois
copyright FEMTO ENGINEERING, 2019-2025
license   GPL v3.0+
brief     Emulation of basic SCPI DMM
"""

import logging
import time
import random
import signalslot as ss

import scinstr.dmm.dmm34461a as dmm34461a

# Only to mime real device import (detection of circular reference)
import socket
import usbtmc


# =============================================================================
class Dmm34461aEmul(dmm34461a.Dmm34461aAbstract):
    """Emulate DMM device connected.
    """

    def __init__(self, *args, **kwargs):
        """The constructor.
        :returns: None
        """
        logging.info("Init DMM test device: %r", self)
        super().__init__()
        for idx, value in enumerate(args):
            logging.info('DMM test device non-keyworded argument %02d: %r',
                         idx, value)
        for key, value in kwargs.items():
            logging.info('DMM test device named argument %r: %r', key, value)
        logging.info("DMM test device %r initialization done", self)

    def connect(self):
        """Connection process to DMM.
        :returns: True if connection success other False (Bool)
        """
        logging.info("Connected to DMM test device: %r", self)
        return True

    @property
    def is_connected(self):
        """Check connection property.
        :returns: True if device connected else False (bool)
        """
        logging.info("Test connection to DMM test device: %r", self)
        return True

    def close(self):
        """Closing process with DMM.
        :returns: None
        """
        logging.info("Connection to DMM test device %r closed", self)

    def get_error(self):
        """Subclass method to emulate response of device.
        """
        return ["+0,\"No error\""]

    @property
    def timeout(self):
        """Gets timeout on socket operations.
        :returns: timeout value in second (float)
        """
        logging.info("Get DMM test device timeout: %r", self._timeout)
        return self._timeout

    @timeout.setter
    def timeout(self, timeout):
        """Sets timeout on socket operations.
        :param timeout: timeout value in second (float)
        :returns: None
        """
        self._timeout = timeout
        logging.info("Set DMM test device timeout: %r", timeout)

    def _write(self, data):
        """Emulate write process
        :param data: data writes to device (str)
        :returns: None
        """
        logging.info("Write %r to DMM test device %r", data, self)

    def _read(self, length):
        """Emulate read process.
        :param length: length of message to read (int)
        :returns: Message reads from device (str)
        """
        data = self._random_gen()
        logging.info("Read %r from DMM test device %r", data, self)
        return data

    def _random_gen(self):
        """Return a random value: use to emulate data input acquisition.
        """
        time.sleep(1.0)
        return "{:+E}".format(random.uniform(-1.0, 1.0))


# =============================================================================
class SDmm34461aEmul(Dmm34461aEmul):
    """Class derived from Dmm34461aEmul class to add signal/slot facilities.
    """

    connected = ss.Signal()
    closed = ss.Signal()
    id_checked = ss.Signal(['flag'])
    out_updated = ss.Signal(['value'])

    def connect(self):
        """Abstract protocol connect process. Derived classes must implement
        the connect process dedicated to the specific protocol used.
        :returns: None
        """
        retval = super().connect()
        if retval is True:
            self.connected.emit()
        return retval

    def close(self):
        """Abstract protocol closing process. Derived classes must implement
        the closing process dedicated to the specific protocol used.
        :returns: None
        """
        super().close()
        self.closed.emit()

    def check_interface(self):
        retval = super().check_interface()
        self.id_checked.emit(flag=retval)
        return retval

    def data_read(self):
        retval = super().data_read()
        if retval is not None:
            self.out_updated.emit(value=retval)
            return retval

    def set_timeout(self, timeout, **kwargs):
        """Sets timeout on operations.
        :param timeout: timeout value in second (float)
        :returns: None
        """
        self.timeout = timeout
        logging.info("Set DMM test device timeout: %r", timeout)

    def get_timeout(self):
        """Gets timeout on socket operations.
        :returns: timeout value in second (float)
        """
        logging.info("Get DMM test device timeout: %r", self.timeout)
        return self.timeout

    def set_pid(self, pid, **kwargs):
        """Set PID used to speak with device through USB.
        :param pid:
        :returns: None
        """
        self.pid = pid
        logging.info("Set DMM test device PID: %r", pid)

    def get_pid(self):
        """Get PID.
        :returns: pid
        """
        logging.info("Get DMM test device PID: %r", self.pid)
        return self.pid

    def set_vid(self, vid, **kwargs):
        """Set VID used to speak with device through USB.
        :param vid:
        :returns: None
        """
        self.vid = vid
        logging.info("Set DMM test device vid: %r", vid)

    def get_vid(self):
        """Get VID.
        :returns: vid
        """
        logging.info("Get DMM test device vid: %r", self.vid)
        return self.vid

    def set_ip(self, ip, **kwargs):
        """Sets IP address used to speak with device.
        :param ip: IP address (str)
        :return: None
        """
        self._ip = ip
        logging.info("Set DMM test device ip: %r", ip)

    def get_ip(self):
        """Gets IP used to speak with device.
        :returns: IP address (str)
        """
        logging.info("Get DMM test device ip: %r", self._ip)
        return self._ip

    def set_port(self, port, **kwargs):
        """Sets internet port used to speak with device.
        :param port: port used by DMM (int)
        :returns: None
        """
        self._port = port
        logging.info("Set DMM test device port: %r", port)

    def get_port(self):
        """Gets internet port used to speak with device.
        :returns: port used by DMM (int)
        """
        logging.info("Get DMM test device port: %r", self._port)
        return self._port


# =============================================================================
def check_dmm():
    """Check the Dmm34461axx class: connect to the multimeter, configures a dc
    voltage measurement then collect and print data to standard output.
    """
    from datetime import datetime

    date_fmt = "%d/%m/%Y %H:%M:%S"
    log_format = "%(asctime)s %(levelname) -8s %(filename)s " + \
                 " %(funcName)s (%(lineno)d): %(message)s"
    logging.basicConfig(level=logging.INFO,
                        datefmt=date_fmt,
                        format=log_format)

    dmm = Dmm34461aEmul(0x2a8d, 0x1601, timeout=4.8)
    # dmm = Dmm34461aEmul(ip="192.168.0.61", port=5025, timeout=2.8)
    if dmm.connect() is not True:
        print("Connection failed")
        return
    dmm.reset()

    print("IDN:", dmm.query("*IDN?"))
    dmm.write("CONF:VOLT:DC AUTO")  # Autorange
    dmm.write("VOLT:DC:NPLC 100")
    dmm.write("VOLT:DC:ZERO:AUTO ON")  # Autozero off
    print("Error config?:", dmm.get_error())

    try:
        while True:
            value = dmm.data_read()
            now = datetime.utcnow()
            if value is None or value == "":
                print("# No data @", now)
            else:
                print(now, value)
    except KeyboardInterrupt:
        dmm.write("ABORT")
    except Exception as er:
        logging.error("# Exception during acquisition: %r", er)

    print("Final error?:", dmm.get_error())

    dmm.close()


# =============================================================================
if __name__ == '__main__':
    check_dmm()



"""package compressor
author  Benoit Dubois
brief   Basic API to handle CP2800 compressor part from Cryomech.
details CP2800 compressor provide  SMDP (Sycon Multi Drop Protocol) over
        RS232/485 interface. See manual for more information.

        Read data packet structure:
        - send:     <'c'><hashval-int><array index-byte>
        - returns:  <'c'><hashval-int><array index-byte><data-long>

        Example, read the dictionary variable COMP_MINUTES (hash 0x454C):
        - send :   0x63|0x45|0x4C|0x00
        - returns: 0x63|0x45|0x4C|0x00|0x00|0x01|0x36|0x23
        Value returned is 00013623(hex) (79395 decimal) minutes.

        Write data packet structure:
        - send:     <'a'><hashval-int><array index-byte><data-long>
        - returns:  None

        Example, start the compressor by writing 1 to EV_START_COMP_REM
        (hash 0xD501):
        - send:    0x61|0xD5|0x01|0x00|0x00|0x00|0x00|0x01
        - returns: None
"""

import serial
import logging
import binascii
from PyQt4.QtCore import QObject, pyqtSignal, pyqtSlot
import smdp


HASHCODE = {
    'CODE_SUM': '\x2B\x0D',
    'MEM_LOSS': '\x80\x1A',
    'CPU_TEMP': '\x35\x74',
    'BATT_OK': '\xA3\x7A',
    'BATT_LOW': '\x0B\x8B',
    'COMP_MINUTES': '\x45\x4C',
    'MOTOR_CURR_A': '\x63\x8B',
    'RI_RMT_COMP_START': '\xBA\xF7',
    'RI_RMT_COMP_STOP': '\x3D\x85',
    'RI_RMT_COMP_ILOK': '\xB1\x5A',
    'RI_SLVL': '\x95\xE3',
    'TEMP_TNTH_DEG': '\x0D\x8F',
    'TEMP_TNTH_DEG_MINS': '\x6E\x58',
    'TEMP_TNTH_DEG_MAXES': '\x8A\x1C',
    'CLR_TEMP_PRES_MMMARKERS': '\xD3\xDB',
    'TEMP_ERR_ANY': '\x6E\x2D',
    'PRES_TNTH_PSI': '\xAA\x50',
    'PRES_TNTH_PSI_MINS': '\x5E\x0B',
    'PRES_TNTH_PSI_MAXES': '\x7A\x62',
    'PRES_ERR_ANY': '\xF8\x2B',
    'H_ALP': '\xBB\x94',
    'H_AHP': '\x7E\x90',
    'H_ADP': '\x31\x9C',
    'H_DPAC': '\x66\xFA',
    'DIODES_UV': '\x8E\xEA',
    'DIODES_TEMP_CDK': '\x58\x13',
    'DIODES_ERR': '\xD6\x44',
    'DCAL_SEL': '\x99\x65',
    'EV_START_COMP_REM': '\xD5\x01',
    'EV_STOP_COMP_REM': '\xC5\x98',
    'COMP_ON': '\x5F\x95',
    'ERR_CODE_STATUS': '\x65\xA4'
    }


#==============================================================================
# CLASS Cp2800
#==============================================================================
class Cp2800(QObject):
    """Class dedicated to handle the CP2800 device.
    """

    COMP_ADDR = '\x10'  # Compressor address to be used to use RS232 interface
    PORT = 0            # Default serial port
    TIMEOUT = 0.5       # Default timeout (in s)
    BAUDRATE = 9600     # Default baudrate (baud/s)
    READ_SIZE = 128     # Number of bytes to read
    READ = '\x63'       # Read data byte 
    WRITE = '\x61'      # Write data byte 
    CMD_RSP = '\x80'    # Command/Response default character

    new_data = pyqtSignal(int)

    def __init__(self, port=PORT, baudrate=BAUDRATE, timeout=TIMEOUT, \
                 addr=COMP_ADDR):
        """Constructor
        :param port: device name or port number value (str or int)
        :param baudrate: baudrate (9600 or 115200) (int)
        :param timeout: timeout on serial interface (int)
        :param addr: address of device (str)
        :returns: None
        """
        super(Cp2800, self).__init__()
        self._addr = addr
        try:
            self._intf = smdp.Smdp(port=port, baudrate=baudrate, \
                                   timeout=timeout)
        except serial.SerialException as ex:
            logging.error("Connection to CP2800 failed: " + str(ex))
        except Exception as ex:
            raise ex

    @pyqtSlot()
    def close(self):
        """Call destructor.
        :returns: None
        """
        self._intf.close()

    @pyqtSlot()
    def open(self):
        """Open connection to device.
        :returns: None
        """
        if self._intf.is_open() is True:
            logging.warning("Port is already open")
            return
        try:
            self._intf.open()
        except serial.SerialException as ex:
            logging.error("Connection to CP2800 failed: " + str(ex))
        except Exception as ex:
            raise ex

    @pyqtSlot(str, str, str)
    def ask(self, keycode, index='\x00', writedata=''):
        """Combines a read after write request and return the result of the
        command.
        :param keycode: key code of data dictionnary of device (str)
        :param index: index of the data (for array) (str)
        :param writedata: data to write (if writing) (str)
        :returns: Results of query (str)
        """
        if keycode not in HASHCODE:
            raise KeyError
        if writedata == '':
            readwrite = self.READ
        else:
            readwrite = self.WRITE
        data = self._gen_data(readwrite, HASHCODE[keycode], index, writedata)
        self._intf.write(self._addr, self.CMD_RSP, data)
        data = self._intf.read(self.READ_SIZE)
        self.new_data.emit(data)
        return data

    def _gen_data(self, readwrite, hashcode, index, writedata):
        """Generate data field of message. More details in doc.
        :param readwrite: read or write data byte (str)
        :param hashcode: hash code of data dictionnary of device (str)
        :param index: index of the data (for array) (str)
        :param writedata: data to write (if writing) (str)
        :returns: the formated data field (str)
        """
        if readwrite == self.READ:
            outlen = 4
        elif readwrite == self.WRITE:
            outlen = 8
        else:
            raise KeyError("readwrite parameter is not valid")
        output = readwrite + hashcode + index + writedata
        if len(output) == outlen:
            return output
        else:
            logging.error("readwrite suggests length %d but gen_data input " \
                          "only %d: %s", outlen, len(output), \
                          binascii.hexlify(output))



# -*- coding: utf-8 -*-

"""package scinstr
author    Benoit Dubois
copyright FEMTO ENGINEERING, 2019
license   GPL v3.0+
brief     API dedicated to handle the Keysight 532X0A counter serie with
          PyQt signal/slot capabilities.
"""

import logging

from PyQt5.QtCore import QObject, pyqtSignal, pyqtSlot

from scinstr.cnt.cnt532x0a import Cnt532x0aEth, Cnt532x0aUsb
import scinstr.cnt.constants as scst


# =============================================================================
class QObjectAdapter(QObject):
    """cf https://rhettinger.wordpress.com/2011/05/26/super-considered-super/
    "How to Incorporate a Non-cooperative Class"
    """
    def __init__(self, **kwargs):
        self._object = QObject(*kwargs)
        super().__init__(**kwargs)


# =============================================================================
class QCnt532x0aUsb(QObjectAdapter, Cnt532x0aUsb):
    """Class derived from Cnt532x0aEth class to add PyQt facilities.
    """

    connected = pyqtSignal()
    closed = pyqtSignal()
    id_checked = pyqtSignal(bool)
    out_updated = pyqtSignal((float,), (str,))

    def __init__(self, vendor_id=scst.CNT532X0A_VID,
                 product_id=scst.CNT532X0A_PID,
                 timeout=scst.CNT532X0A_TIMEOUT):
        QObject.__init__(self)
        Cnt532x0aUsb.__init__(self, vendor_id, product_id, timeout)

    @pyqtSlot(object)
    def connect(self, **kwargs):
        """Connection process.
        :returns: True if connection is ok else False (bool)
        """
        retval = Cnt532x0aUsb.connect(self, **kwargs)
        if retval is True:
            self.connected.emit()
        return retval

    @pyqtSlot()
    def close(self):
        """Closing process.
        :returns: None
        """
        Cnt532x0aUsb.close(self)
        self.closed.emit()

    @pyqtSlot()
    def check_interface(self):
        retval = Cnt532x0aUsb.check_interface(self)
        self.id_checked.emit(retval)
        return retval

    @pyqtSlot()
    def data_read(self):
        retval, errors = Cnt532x0aUsb.data_read(self)
        if retval is not None:
            self.out_updated[str].emit(str(retval))
            self.out_updated[float].emit(retval)
            return retval, errors

    @pyqtSlot(float)
    def set_timeout(self, timeout):
        """Sets timeout on operations.
        :param timeout: timeout value in second (float)
        :returns: None
        """
        self.timeout = timeout

    @pyqtSlot()
    def get_timeout(self):
        """Gets timeout on socket operations.
        :returns: timeout value in second (float)
        """
        return self.timeout

    @pyqtSlot(str)
    def set_pid(self, pid):
        """
        :param pid:
        :returns: None
        """
        self.pid = pid

    @pyqtSlot()
    def get_pid(self):
        """
        :returns: pid
        """
        return self.pid

    @pyqtSlot(str)
    def set_vid(self, vid):
        """
        :param vid:
        :returns: None
        """
        self.vid = vid

    @pyqtSlot()
    def get_vid(self):
        """
        :returns: vid
        """
        return self.vid


# =============================================================================
class QCnt532x0aEth(QObjectAdapter, Cnt532x0aEth):
    """Class derived from Cnt532x0aEth class to add PyQt facilities.
    """

    connected = pyqtSignal()
    closed = pyqtSignal()
    id_checked = pyqtSignal(bool)
    out_updated = pyqtSignal((float,), (str,))

    def __init__(self, ip=None,
                 port=scst.CNT532X0A_PORT,
                 timeout=scst.CNT532X0A_TIMEOUT):
        QObject.__init__(self)
        Cnt532x0aEth.__init__(self, ip, port, timeout)

    @pyqtSlot(object)
    def connect(self, **kwargs):
        """Abstract protocol connect process. Derived classes must implement
        the connect process dedicated to the specific protocol used.
        :returns: None
        """
        retval = Cnt532x0aEth.connect(self, **kwargs)
        if retval is True:
            self.connected.emit()
        return retval

    @pyqtSlot()
    def close(self):
        """Abstract protocol closing process. Derived classes must implement
        the closing process dedicated to the specific protocol used.
        :returns: None
        """
        Cnt532x0aEth.close(self)
        self.closed.emit()

    @pyqtSlot()
    def check_interface(self):
        retval = Cnt532x0aEth.check_interface(self)
        self.id_checked.emit(retval)
        return retval

    @pyqtSlot()
    def data_read(self):
        retval, errors = Cnt532x0aEth.data_read(self)
        if retval is not None:
            self.out_updated[str].emit(str(retval))
            self.out_updated[float].emit(retval)
            return retval, errors

    @pyqtSlot(float)
    def set_timeout(self, timeout):
        """Sets timeout on socket operations.
        :param timeout: timeout value in second (float)
        :returns: None
        """
        self._sock.settimeout(timeout)

    @pyqtSlot()
    def get_timeout(self):
        """Gets timeout on socket operations.
        :returns: timeout value in second (float)
        """
        return self._sock.gettimeout()

    @pyqtSlot(str)
    def set_ip(self, ip):
        """Sets IP address used to speak with device.
        :param ip: IP address (str)
        :return: None
        """
        self._ip = ip

    @pyqtSlot()
    def get_ip(self):
        """Gets IP used to speak with device.
        :returns: IP address (str)
        """
        return self._ip

    @pyqtSlot(int)
    def set_port(self, port):
        """Sets internet port used to speak with device.
        :param port: port used by counter532x0a (int)
        :returns: None
        """
        self._port = port

    @pyqtSlot()
    def get_port(self):
        """Gets internet port used to speak with device.
        :returns: port used by counter532x0a (int)
        """
        return self._port


# =============================================================================
def check_counter532x0a():
    """Check the Counter532x0a class: connect to the multimeter, configure a dc
    voltage measurement then collect and print data to standard output.
    Values that work:
    - 1PLC with 0.5 de rtimeout
    - 10PLC with 0.8 de rtimeout
    """
    from datetime import datetime
    # from mjdutil import datetime_to_mjd

    date_fmt = "%d/%m/%Y %H:%M:%S"
    log_format = "%(asctime)s %(levelname) -8s %(filename)s " + \
                 " %(funcName)s (%(lineno)d): %(message)s"
    logging.basicConfig(level=logging.INFO,
                        datefmt=date_fmt,
                        format=log_format)

    counter = QCnt532x0aUsb(0x0957, 0x1707, timeout=2.8)
    # counter = Counter532x0aEth(ip="192.168.0.20", port=5025, timeout=1.5)
    if counter.connect() is not True:
        print("Connection failed")
        return
    counter.reset()
    print("IDN:", counter.idn())
    counter.write("CONF:FREQ 100.0E6")
    counter.write("TRIG:SLOP POS")
    counter.write("SENS:FREQ:GATE:TIME 1.0")
    counter.write("SENS:FREQ:GATE:SOUR TIME")
    print("Error config?:", counter.check_error())

    try:
        while True:
            value = counter.data_read()
            now = datetime.utcnow()
            if value is None or value == "":
                print("# No data @", now)
            else:
                # print(now, '{:0.14f}'.format(value.strip('\n')))
                print(now, value)
    except KeyboardInterrupt:
        pass
    except Exception as er:
        print("# Exception during acquisition: " + str(er))
    counter.reset()
    counter.close()


# =============================================================================
if __name__ == '__main__':
    check_counter532x0a()



# -*- coding: utf-8 -*-

"""package mercury
author    Benoit Dubois
copyright FEMTO ENGINEERING, 2019
license   GPL v3.0+
brief     Class to handle Mercury Itc device through its different interfaces
          (USB, Ethernet, Serial).
"""

import time
import logging
import serial
import socket
import pysobus
from pysobus.parser import Parser
import usb.core
import usb.util
import usb.control


# =============================================================================
class MercuryItc(object):
    """Base class to handle Mercury Itc device.
    """

    def connect(self):
        """Abstract protocol connect process. Derived classes must implement
        the connect process dedicated to the specific protocol used.
        :returns: None
        """
        raise NotImplementedError("Method not implemented by derived class")

    def close(self):
        """Abstract protocol closing process. Derived classes must implement
        the closing process dedicated to the specific protocol used.
        :returns: None
        """
        raise NotImplementedError("Method not implemented by derived class")

    def _write(self, data):
        """Abstract protocol write process. Derived classes must implement
        the write process dedicated to the specific protocol used.
        :param data: data writes to device (str)
        :returns: None
        """
        raise NotImplementedError("Method not implemented by derived class")

    def _read(self, length):
        """Abstract protocol read process. Derived classes must implement
        the read process dedicated to the specific protocol used.
        :param length: length of message to read (int)
        :returns: Message reads from device (str)
        """
        raise NotImplementedError("Method not implemented by derived class")

    def read(self, length=100):
        """A basic read method: read a message from device.
        :param length: length of message to read (int)
        :returns: Message reads from device (str)
        """
        try:
            retval = self._read(length)
        except Exception as ex:
            logging.error("Read error: %r", ex)
            return ''
        logging.debug("read: %r", retval)
        return retval

    def write(self, data):
        """A basic write method: writes "data" to device.
        :param data: data writes to device (str)
        :returns: None
        """
        try:
            nb_bytes = self._write(data)
        except Exception as ex:
            logging.error("Write error: %r", ex)
            return 0
        logging.debug("write: %r", data)
        return nb_bytes

    def query(self, data, length=1024):
        """Write than read procedure.
        :param data: data writes to device (str)
        :returns: Message returned by device (str)
        """
        self.write(data)
        return self.read(length)

    @property
    def idn(self):
        """Return product id of device.
        :returns: product id of device (str)
        """
        self.write("*IDN?")
        _id = self.read(100)
        return _id


#==============================================================================
class MercuryItcEth(MercuryItc):
    """Handle Mercury iTC device through ethernet interface.
    """

    PORT = 7020
    TIMEOUT = 3.0  # Default timeout in second

    def __init__(self, ip='', port=PORT, timeout=TIMEOUT):
        self._sock = None
        self.ip = ip
        self.port = port
        self.timeout = timeout

    def connect(self):
        """Open connection with device.
        :param ip: IP address of device (str)
        :param port: Device port in use (int)
        :param timeout: Timeout in second (float)
        :returns: True if connection succeed False elsewhere (bool)
        """
        self._sock = socket.socket(socket.AF_INET,
                                   socket.SOCK_STREAM,
                                   socket.IPPROTO_TCP)
        self._sock.settimeout(self.timeout)
        try:
            self._sock.connect((self.ip, self.port))
        except ValueError as ex:
            logging.error("Wrong connection parameters: %r", ex)
            return False
        except socket.timeout:
            logging.error("Timeout on connection")
            return False
        logging.info("Connected to Lakeshore")
        return True

    def close(self):
        """Close connection with device.
        :returns: None
        """
        if self._sock is None:
            return
        self._sock.close()
        self._sock = None
        logging.info("Connection to Lakeshore closed")

    def _write(self, data):
        """Specific ethernet writing process.
        :param data: data writes to device (str)
        :returns: number of bytes sent (int)
        """
        return self._sock.send((data + '\n').encode('utf-8'))

    def _read(self, length):
        """Specific ethernet reading process.
        :param length: length of message to read (int)
        :returns: Message reads from device (str)
        """
        return self._sock.recv(length).decode('utf-8').strip('\n')


#==============================================================================
class MercuryItcUsb(MercuryItc):

    VID = 0x0525
    PID = 0xa4a7
    TIMEOUT = 2.0

    _CONFIG_ID     = 2
    #
    _INTERFACE_ID0  = 0       # The interface we use to talk to the device
    _ALT_INTF_ID0   = 0       # The alternate interface use to talk to the device
    _IN_DAT_EP_ID  = 5       # EP8 @0x83 = 131    -> large endpoint
    #
    _INTERFACE_ID1  = 1       # The interface we use to talk to the device
    _ALT_INTF_ID1   = 0       # The alternate interface use to talk to the device
    _IN_EP_ID      = 0       # EP1_IN @0x81 = 129 -> small endpoint
    _OUT_EP_ID     = 1       # EP1_OUT @0x02 = 2  -> small endpoint


    def __init__(self, vendor_id=VID, product_id=PID, timeout=TIMEOUT):
        self._dev = None
        self.vid = vendor_id
        self.pid = product_id
        self._timeout = timeout

    def reset(self):
        if self._dev is not None:
            self._dev.reset()

    def close(self):
        """There is no "concept" of device connection either in USB spec or
        in PyUSB, you can just release resources allocated by the object.
        """
        if self._dev is not None:
            usb.util.dispose_resources(self._dev)

    def connect(self):
        """Set an object representing our device (usb.core.Device).
        :returns: True if connection is Ok else returns False (bool)
        """
        # Find our device
        self._dev = usb.core.find(idVendor=self.vid, idProduct=self.pid)
        print("type(self._dev):", type(self._dev))
        # Was it found?
        if self._dev is None:
            logging.warning("Device not found")
            return False
        # Detach the kernel driver if it is active
        """if self._dev.is_kernel_driver_active(0):
            try:
                self._dev.detach_kernel_driver(0)
            except usb.core.USBError as ex:
                logging.critical("Could not detach kernel driver: %r", ex)
                return False
        self._dev.reset()"""
        self._config(self._CONFIG_ID, self._INTERFACE_ID1, self._ALT_INTF_ID1, self._OUT_EP_ID ,self._IN_EP_ID)
        return True

    def is_connected(self):
        if self._dev is None:
            return False
        return True

    def _config(self, config_id, interface_id, alt_interface_id, out_ep_id,
               in_ep_id) : ##, out_dat_ep_id, in_dat_ep_id):
        """Configure device and get endpoint instance.
        Endpoint Address description:
        - Bits 0..3 Endpoint Number.
        - Bits 4..6 Reserved. Set to Zero
        - Bits 7 Direction 0 = Out, 1 = In (Ignored for Control Endpoints)
        Example: address = 129 = (81)8 = (1000 0001)2
                 -> Direction: In, Number: 1
        :param config_id:
        ...
        :returns: None
        """
        try:
            self._dev.set_configuration(config_id)
        except usb.core.USBError as ex:
            logging.critical("USB error during dev configuration: %r", ex)
            raise
        except Exception as ex:
            logging.critical("Could not set interface configuration: %r", ex)
            raise
        self._dev.reset()
        # Set interface
        try:
            self._dev.set_interface_altsetting(interface=interface_id,
                                               alternate_setting=alt_interface_id)
        except usb.core.USBError as ex:
            logging.critical("USB error during interface altsetting: %r", ex)
            raise
        except Exception as ex:
            logging.critical("Error during interface altsetting: %r", ex)
            raise
        # Get endpoint instances
        try:
            self._ep_out = self._dev[config_id-1] \
                [(interface_id, alt_interface_id)][out_ep_id]
            self._ep_in = self._dev[config_id-1] \
                [(interface_id, alt_interface_id)][in_ep_id]
            """self._ep_dat_out = self._dev[config_id-1][(interface_id, \
                                            alt_interface_id)][out_dat_ep_id]
            self._ep_dat_in = self._dev[config_id-1][(interface_id, \
                                        alt_interface_id)][in_dat_ep_id]"""
        except usb.core.USBError as ex:
            logging.critical("Could not set endpoint setting: %r", ex)
        logging.debug("USB interface succefuly configured")

    def write_ctrl(self, msg):
        """Write raw data on control endpoint.
        The method don't take care about meaning of msg.
        :param msg: message to write to device (sequence like type convertible
                    to array type of int cf usb.core.device.write()).
        :returns: number of bytes sent (int).
        """
        try:
            nb = self._ep_out.write(msg)
        except usb.core.USBError as ex:
            logging.error("Could not write data: %r", ex)
            return 0
        return nb

    def read_ctrl(self):
        """Read raw dat on control endpointa.
        :returns: Received bytes (list of ?).
        """
        # Collect data
        try:
            data = self._ep_in.read(self._ep_dat_in.wMaxPacketSize)
        except usb.core.USBError as ex:
            logging.error("Could not read data: %r", ex)
        return data

    def _write(self, msg):
        """Write raw data.
        The method don't take care about meaning of msg.
        Write on the DDS board is handled by a FX2 device (USB interface).
        A write cycle on the FX2 device is divided in two parts:
        - init message writed to main endpoint (EP0),
        - real message writed to 'data' endpoint (EP4).
        Writing on the USB bus is transparent: you transmit each bit in order.
        Writing on the USB bus consists in filling the write function (usb.core)
        with a list of binary values.
        :param msg: message to write to device (sequence like type convertible
                    to array type of int cf usb.core.device.write()).
        :returns: number of bytes sent (int).
        """
        try:
            nb = self._ep_dat_out.write(msg)
        except usb.core.USBError as ex:
            logging.error("Could not write data: %r", ex)
            return 0
        return nb

    def _read(self):
        """Read raw data.
        The method don't take care about meaning of data.
        :returns: Received bytes (list of ?).
        """
        # Collect data
        try:
            data = self._ep_dat_in.read(self._ep_dat_in.wMaxPacketSize)
        except usb.core.USBError as ex:
            logging.error("Could not read data: %r", ex)
        return data

    @property
    def timeout(self):
        """Gets timeout on socket operations.
        :returns: timeout value in second (float)
        """
        if self._dev is None:
            logging.debug("Can't get timeout value, device not connected")
            return None
        return self._dev.timeout

    @timeout.setter
    def timeout(self, timeout):
        """Sets timeout on socket operations.
        :param timeout: timeout value in second (float)
        :returns: None
        """
        if self._dev is None:
            logging.debug("Can't set timeout valule, device not connected")
            return
        self._dev.timeout = timeout
        self._timeout = timeout


#==============================================================================
class MercuryItcRs232(MercuryItc):

    """ISOBUS_ADDRESS = 1
    """

    BAUD = 9600
    DATA_SIZE = serial.EIGHTBITS
    PARITY = serial.PARITY_NONE
    STOP_BITS = serial.STOPBITS_ONE
    FLOW_CONTROL = False
    TIMEOUT = 2.0

    def __init__(self, port="", baudrate=BAUD, bytesize=DATA_SIZE,
                 parity=PARITY, stopbits=STOP_BITS, flowctrl=FLOW_CONTROL,
                 timeout=TIMEOUT):
        super().__init__()
        self._ser = None
        self.port = port
        self.baudrate = baudrate
        self.bytesize = bytesize
        self.parity = parity
        self.stopbits = stopbits
        self.timeout = timeout

    def connect(self):
        """Connect to the remote host
        :returns: True if connection succeeded, False otherwise
        """
        self._ser = serial.Serial(self.port,
                                  baudrate=self.baudrate,
                                  bytesize=self.bytesize,
                                  parity=self.parity,
                                  stopbits=self.stopbits,
                                  timeout=self.timeout)
        if self._ser.isOpen() is False:
            try:
                self._ser.open()
            except ValueError as ex:
                logging.error("Wrong connection parameters: %r", ex)
                return False
            except serial.SerialTimeoutException:
                logging.error("Timeout on connection")
                return False
        logging.info("Connected to device")
        self._ser.timeout = self.timeout
        self._ser.flush()
        return True

    def close(self):
        """Closes the underlying serial connection
        """
        if self._ser is not None:
            try:
                self._ser.close()
            except Exception as ex:
                logging.error("Error when closing USB connection: %r", ex)
        self._ser = None
        logging.info("Connection to device closed")

    def _write(self, data):
        """Specific USB writing process.
        :param data: data writes to device (str)
        :returns: number of bytes sent (int)
        """
        try:
            nb_bytes = self._ser.write(data.encode('utf-8'))
        except serial.SerialTimeoutException:
            logging.error("Device write timeout")
            return 0
        except Exception as ex:
            logging.error("Device write error %r", ex)
            return 0
        return nb_bytes

    def _read(self, length):
        """Specific USB reading process.
        :param length: length of message to read (int)
        :returns: Message reads from device (str)
        """
        try:
            retval = self._ser.read(length).decode('utf-8')
        except serial.SerialTimeoutException:
            logging.error("Device read timeout")
            return ''
        except Exception as ex:
            logging.error("Device read error %r", ex)
            return ''
        return retval


#==============================================================================
if __name__ == "__main__":
    # Setup logger
    LOG_FORMAT = '%(asctime)s %(levelname)s %(filename)s (%(lineno)d): ' \
        +'%(message)s'
    logging.basicConfig(format=LOG_FORMAT, level=logging.DEBUG)

    DEV = MercuryItcEth("192.168.0.22", timeout=5.0)
    #DEV = MercuryItcUsb()
    #DEV = MercuryItcRs232("/dev/ttyACM0")

    if DEV.connect() is False:
        exit()
    print("Connected to ", DEV.idn)




# -*- coding: utf-8 -*-

"""package stanford
author    Benoit Dubois
copyright FEMTO ENGINEERING, 2018
license   GPL v3.0+
brief     Class for handling SR8x0 device through ethernet connection.
details   Ethernet connection is provided through Prologix GPIB-Ethernet
          adapter.
"""

from iopy.prologix import PrologixGpibEth
import scinstr.lockin.sr8x0 as sr8x0


class Sr8x0Ethernet(sr8x0.Sr8x0):
    """Stanford Research System SR8x0 ethernet (prologix GPIB-Ethernet adapter)
    interface.
    """

    PORT = 1234

    def __init__(self, ip='', port=PORT, gpib_addr=1):
        self._intf = None
        self.ip = ip
        self.port = port
        self.gpib_addr = gpib_addr

    def connect(self):
        """Open connection with device.
        :param ip: IP address of device (str)
        :param port: Device port in use (int)
        :param timeout: Timeout in second (float)
        :returns: True if connection succeed False elsewhere (bool)
        """
        self._dev = PrologixGpibEth(self.ip, self.port, self.gpib_addr)
        try:
            self._dev.connect((self.ip, self.port))
        except ValueError as ex:
            logging.error("Wrong connection parameters: %r", ex)
            return False
        except Exception as ex:
            logging.error("Exception during connection: %r", ex)
            return False
        logging.info("Connected to SR8x0")
        return True

    def close(self):
        """Close connection with device.
        :returns: None
        """
        if self._dev is None:
            return
        self._dev.close()
        self._dev = None
        logging.info("Connection to SR8x0 closed")

    def write(self, data):
        """Specific ethernet writing process.
        :param data: data writes to device (str)
        :returns: number of bytes sent (int)
        """
        try:
            nb_bytes = self._dev.write((data + '\n').encode('utf-8'))
        except Exception as ex:
            logging.error("Device write error %r", ex)
            return 0
        logging.debug("write %r", data)
        return nb_bytes

    def read(self, length):
        """Specific ethernet reading process.
        :param length: length of message to read (int)
        :returns: Message reads from device (str)
        """
        try:
            retval = (self._dev.read(length).decode('utf-8')).strip('\n')
        except Exception as ex:
            logging.error("Device read error %r", ex)
            return ''
        logging.debug("read %r", retval)
        return retval


# =============================================================================
if __name__ == '__main__':
    import logging
    import argparse

    # Handles log
    DATE_FMT = "%d/%m/%Y %H:%M:%S"
    LOG_FORMAT = "%(asctime)s %(levelname) -8s %(filename)s " + \
                 " %(funcName)s (%(lineno)d): %(message)s"
    logging.basicConfig(level=logging.DEBUG,
                        datefmt=DATE_FMT,
                        format=LOG_FORMAT)

    parser = argparse.ArgumentParser(description='Lakeshore Monitoring')
    parser.add_argument('--ip', dest='ip', type=str,
                        help='If interface = \'ethernet\', IP of device')

    # Parse command line
    args = parser.parse_args()
    ip = args.ip

    INST = Sr8x0Ethernet(ip=ip, timeout=1.0)
    INST.connect()

    print("IDN: %s" % INST.idn)

    print("Single shot acquisition")
    INST.reset_data_storage()
    INST.sample_rate = sr8x0.SAMPLE_RATES["256 Hz"]
    INST.single_shot = 1
    INST.start_data_storage()
    INST.wait_bit0()
    DATA = INST.read_buffer(_format='a')
    with open("sr810_data.dat", 'w') as fd:
        fd.write("# Sample rate: " + INST.sample_rate)
        for i in DATA:
            fd.write("%e\n" % i)
    INST.close()



# -*- coding: utf-8 -*-

"""package scinstr
author    Benoit Dubois
copyright FEMTO ENGINEERING, 2026
license   LGPL v3.0+
brief     Service class (hardware communication interface) for handling
          NF LI56[45|50] devices
"""

import socket
import serial
import pyvisa
from typing import Protocol


# ----------------------------------------------------------------------------
class HwIntfService(Protocol):

    def connect(self) -> bool:
        """Connect to the device
        :returns: True if connection succeeded, False otherwise
        """
        ...

    def close(self) -> None:
        """Closes the underlying connection"""
        ...

    def write(self, msg: str) -> int:
        """Writing process.
        :param msg: message writes to device (str)
        :returns: number of bytes sent (int)
        """
        ...

    def read(self, length: int) -> str:
        """Reading process.
        :param length: length of message to read (int)
        :returns: Message reads from device (str)
        """
        ...

    def query(self, msg: str) -> str:
        """Basic query i.e. write then read.
        """
        ...


# ----------------------------------------------------------------------------
class Usb(HwIntfService):

    def __init__(
        self, vendor_id: int, product_id: int, serial_id: str = "", timeout: float = 0
    ) -> None:
        self.vid = vendor_id
        self.pid = product_id
        self.sid = serial_id
        self._timeout = timeout  # in second

    def connect(self) -> bool:
        rm = pyvisa.ResourceManager()
        # USB[board]::manufacturer ID::model code::serial number[::USB interface number][::INSTR]
        # USB::0x1234::125::A22-5::INSTR
        try:
            self._dev = rm.open_resource(
                f"USB::{self.vid}::{self.pid}::{self.sid}::INSTR"
            )
        except pyvisa.VisaIOError as e:
            print(f"Opening USB interface failed: {e}")
            return False
        self._dev.read_termination = "\n"
        self._dev.write_termination = "\n"
        self._dev.timeout = self._timeout * 1000  # in ms
        return True

    def close(self) -> None:
        self._dev.close()

    def write(self, msg: str) -> int:
        try:
            nb_byte = self._dev.write(msg)
        except AttributeError as ex:  # Exception raised after first write
            return 0
        return nb_byte

    def read(self, length: int) -> str:
        return self._dev.read()

    def query(self, msg: str) -> str:
        return self._dev.query(msg)


# ----------------------------------------------------------------------------
class Serial(HwIntfService):

    def __init__(
        self, port: str, baudrate: int = 9600, timeout: float = 0.5
    ) -> None:
        self.port = port
        self._timeout = timeout
        self._baudrate = baudrate
        self._dev = None

    def connect(self) -> bool:
        try:
            self._dev = serial.Serial(
                port=self.port, 
                baudrate=self._baudrate,
                timeout=self._timeout
                )
        except Exception as ex:
            print(f"Opening serial port failed: {ex}")
            return False
        return True

    def close(self) -> None:
        if self._dev:
            self._dev.close()

    def write(self, msg: str) -> int:
        return self._dev.write((msg + '\n').encode("utf-8"))

    def read(self, length: int) -> str:
        return self._dev.read(length).decode("utf-8")

    def query(self, msg: str) -> str:
        self._dev.write((msg + '\n').encode("utf-8"))
        return self._dev.readline().decode("utf-8")[:-1]


# ----------------------------------------------------------------------------
class Socket(HwIntfService):

    def __init__(self, ip: str, port: int, timeout: float = 0.1) -> None:
        """
        :param ip: IP address of device (str)
        :param port: Ethernet port of device (int)
        :param timeout: Socket timeout value in s (float)
        :returns: None
        """
        self._sock = None
        self._ip = ip
        self._port = port
        self._timeout = timeout

    def connect(self):
        self._sock = socket.socket(
            socket.AF_INET, socket.SOCK_STREAM, socket.IPPROTO_TCP
        )
        self._sock.settimeout(self._timeout)
        try:
            self._sock.connect((self._ip, self._port))
        except Exception as ex:
            print(f"Opening socket failed: {ex}")
            return False
        return True

    def close(self):
        self._sock.close()

    def write(self, msg):
        self._sock.send(msg.encode("utf8"))

    def read(self, length):
        return self._sock.recv(length).decode("utf-8")



# -*- coding: utf-8 -*-

"""package scinstr
author    Benoit Dubois
copyright FEMTO ENGINEERING, 2019-2025
license   GPL v3.0+
brief     Emulated DMM with state tuningable from webserver
"""

import logging
import time
import threading
import random
from enum import Enum
from remi import start, App, gui

# Only to mime real device import (detection of circular reference)
import socket
import usbtmc


ACQUISITION_TIMES = {
    "1": 0.02,
    "10": 0.2,
    "100": 2,
}

DEFAULT_PORT = 8090
DEFAULT_FREQUENCY = 10E3
DEFAULT_VOLTAGE = 0.0005
DEFAULT_RANDOM_RANGE = 0.0001
DEFAULT_RANDOM_STATE= True
DEFAULT_ENABLE_PERIOD = 15
DEFAULT_DISABLE_PERIOD = 5
DEFAULT_ACQUISITION_TIME = ACQUISITION_TIMES["10"]


DmmMode= Enum('Mode', [("VOLTMETER", 1), ("FREQUENCYMETER", 2)])
CommunicationState = Enum('State', [("OPENED", 1), ("CLOSED", 2), ("FAULTED", 3)])

DEFAULT_MODE = DmmMode.VOLTMETER


# =============================================================================
def mystart(class_, port, userdata):
    start(class_, port=port, userdata=userdata)


# =============================================================================
class RepeatTimer(threading.Timer):
    """From https://stackoverflow.com/questions/12435211/threading-timer-repeat-function-every-n-seconds
    """
    def run(self)-> None:
        while not self.finished.wait(self.interval):
            self.function(*self.args, **self.kwargs)


# =============================================================================
class DmmEmulator:
    """Emulate a basic DMM device behavior and problem:
    - select mode between frequencymeter and voltmeter
    - get (emulated) measured values (user defined or randomized)
    - emulate disconnection/reconnection with device
    """

    def __init__(self, *args, **kwargs)-> None:
        """The constructor.
        :returns: None
        """
        self._log = logging.getLogger('scinstr.dmm.emul')
        self._communication_state = CommunicationState.CLOSED
        self._com_fail_duration = {'enable':DEFAULT_ENABLE_PERIOD, 'disable': DEFAULT_DISABLE_PERIOD}
        self._com_period_timer = None
        self._mode = DEFAULT_MODE
        self._is_random_data = False
        self._random_range = DEFAULT_RANDOM_RANGE
        self._freq = DEFAULT_FREQUENCY
        self._volt = DEFAULT_VOLTAGE
        self._acquisition_time = DEFAULT_ACQUISITION_TIME
        for idx, value in enumerate(args):
            self._log.info('DMM emulator device non-keyworded argument %02d: %r',
                         idx, value)
        for key, value in kwargs.items():
            self._log.info('DMM emulator device named argument %r: %r', key, value)
        mythread = threading.Thread(target=mystart, args=(DmmEmulatorHandler, DEFAULT_PORT, (self,)))
        mythread.start()
        self._log.info("DMM emulator device %r initialization done", self)

    def set_frequency(self, value: float) -> None:
        self._freq = value
        self._log.info(f"set_frequency {value}")

    def set_voltage(self, value: float)-> None:
        self._volt = value
        self._log.info(f"set_voltage {value}")

    def set_random_range(self, value: float) -> None:
        self._random_range = value
        self._log.info(f"set_random_range {value}")

    def set_communication_state(self, value: CommunicationState)-> None:
        self._communication_state = value
        self._log.info(f"set_communication_state {value}")

    def _emul_timed_faulted_connection(self, duration: float)-> None:
        """Emulate a faulted connection with device of 'duration' second.
        """
        self._log.info("Begin connection fault test")
        self._communication_state = CommunicationState.FAULTED
        time.sleep(duration)
        self._communication_state = CommunicationState.OPENED
        self._log.info("End connection fault test")

    def set_communication_faillure_params(self, enable_period=DEFAULT_ENABLE_PERIOD, disable_period=DEFAULT_DISABLE_PERIOD)-> None:        
        self._com_fail_duration['enable'] = enable_period
        self._com_fail_duration['disable'] = disable_period

    def set_communication_faillure_state(self, value: bool)-> None:
        if value is False:
            if self._com_period_timer:
                self._com_period_timer.cancel()
            return
        self._com_period_timer = RepeatTimer(
            self._com_fail_duration['enable'], 
            self._emul_timed_faulted_connection, 
            args=(self._com_fail_duration['disable'],)
        )
        self._com_period_timer.start()

    def set_random_data_state(self, state)-> None:
        self._is_random_data = state

    @property
    def timeout(self)-> float:
        """Gets timeout on socket operations.
        :returns: timeout value in second (float)
        """
        self._log.info("Get DMM emulator device timeout: %r", self._timeout)
        return self._timeout

    @timeout.setter
    def timeout(self, timeout)-> None:
        """Sets timeout on socket operations.
        :param timeout: timeout value in second (float)
        :returns: None
        """
        self._timeout = timeout
        self._log.info("Set DMM emulator device timeout: %r", timeout)

    def connect(self)-> bool:
        """Overload connection process to DMM.
        :returns: True if connection success other False (Bool)
        """
        self._communication_state = CommunicationState.OPENED
        self._log.info("Connected to DMM emulator device: %r", self)
        return True

    def close(self)-> None:
        """Closing process with DMM.
        :returns: None
        """
        self._communication_state = CommunicationState.CLOSED
        self._log.info("Connection to DMM emulator device %r closed", self)

    def reset(self) -> bool:
        """Connection process to DMM.
        :returns: True if connection success other False (Bool)
        """
        self._log.info("Reset DMM test device: %r", self)
        return True

    def get_error(self)-> list[str]:
        """Subclass method to emulate response of device.
        """
        return ["+0,\"No error\""]

    @property
    def is_connected(self)-> bool:
        """Check connection property.
        :returns: True if device connected else False (bool)
        """
        match self._communication_state:
            case CommunicationState.OPENED:
                return True
            case CommunicationState.CLOSED:
                return False
            case _:  # i.e. CommunicationState.FAULTED
                raise ConnectionError("Connection with DMM emulator failed")

    def write(self, data)-> int:
        """Emulate write process
        :param data: data writes to device (str)
        :returns: None
        """
        if self._communication_state == CommunicationState.FAULTED:
            raise ConnectionError("Communication with (emulated) DMM failled")
        if "CONF:FREQ" in data:
            self._mode = DmmMode.FREQUENCYMETER
        elif "CONF:VOLT:DC" in data:
            self._mode = DmmMode.VOLTMETER
        elif ":NPLC" in data:
            nplc = data.split("NPLC")[-1].replace(' ', '')
            self._acquisition_time = float(ACQUISITION_TIMES[nplc])
        self._log.info("Write %r to DMM emulator device %r", data, self)
        return len(data)

    def ask(self, msg):
        if "READ?" in msg:
            return self.data_read()
        return msg
        
    def data_read(self)-> float:
        if self._communication_state == CommunicationState.FAULTED:
            raise ConnectionError("Communication with (emulated) DMM failled")
        time.sleep(self._acquisition_time)  # Acquisition time
        if self._mode == DmmMode.FREQUENCYMETER:
            data = self._freq
        else:
            data = self._volt
        if self._is_random_data is True:
            data += self._random_gen()
        return data

    def read(self, length)-> str:
        """Emulate read process.
        :param length: length of message to read (int)
        :returns: Message reads from device (str)
        """
        if self._communication_state == CommunicationState.FAULTED:
            raise ConnectionError("Communication with (emulated) DMM failled")
        data = "emulated_dmm"
        self._log.info("Read %r from DMM emulator device %r", data, self)
        return data

    def _random_gen(self)-> float:
        """Return a random value: use to emulate data input acquisition.
        """
        return random.uniform(-self._random_range / 2.0, self._random_range / 2.0)


# =============================================================================
class DmmEmulatorHandler(App):
    """Handle emulated DMM state from a web app.
    """

    def main(self, dmm)-> gui.Widget:
        self._dmm = dmm
        #
        main_box = gui.VBox()
        title_box = gui.HBox()
        title_lbl = gui.Label('DMM emulator handler', width="80%", margin="5px")
        comm_fault_box = gui.HBox()
        comm_fault_lbl = gui.Label('Emulate communication fault', width="80%", margin="5px")
        self.comm_fault_cbox = gui.CheckBox(checked=False, width="20%", margin="5px")
        randomize_box = gui.HBox()
        randomize_lbl = gui.Label('Randomize value', width="80%", margin="5px")
        self.randomize_cbox = gui.CheckBox(checked=DEFAULT_RANDOM_STATE, width="20%", margin="5px")
        random_range_box = gui.HBox()
        random_range_lbl = gui.Label('Random range (Hz or V)', width="20%", margin="5px")
        self.random_range_input = gui.TextInput(width="40%", margin="5px")
        self.random_range_input.set_text(str(DEFAULT_RANDOM_RANGE))
        self.random_range_btn = gui.Button('Set', width="10%", margin="5px")
        freq_box = gui.HBox()
        freq_lbl = gui.Label('Frequency value (Hz)', width="20%", margin="5px")
        self.freq_input = gui.TextInput(width="40%", margin="5px")
        self.freq_input.set_text(str(DEFAULT_FREQUENCY))
        self.freq_btn = gui.Button('Set', width="10%", margin="5px")
        volt_box = gui.HBox()
        volt_lbl = gui.Label('Voltage value (V)', width="20%", margin="5px")
        self.volt_input = gui.TextInput(width="40%", margin="5px")
        self.volt_input.set_text(str(DEFAULT_VOLTAGE))
        self.volt_btn = gui.Button('Set', width="10%", margin="5px")
        #
        title_box.append(title_lbl)
        comm_fault_box.append(comm_fault_lbl)
        comm_fault_box.append(self.comm_fault_cbox)
        freq_box.append(freq_lbl)
        freq_box.append(self.freq_input)
        freq_box.append(self.freq_btn)
        volt_box.append(volt_lbl)
        volt_box.append(self.volt_input)
        volt_box.append(self.volt_btn)
        randomize_box.append(randomize_lbl)
        randomize_box.append(self.randomize_cbox)
        random_range_box.append(random_range_lbl)
        random_range_box.append(self.random_range_input)
        random_range_box.append(self.random_range_btn)
        main_box.append(title_box)
        main_box.append(comm_fault_box)
        main_box.append(freq_box)
        main_box.append(volt_box)
        main_box.append(randomize_box)
        main_box.append(random_range_box)
        #
        self.comm_fault_cbox.onclick.do(self.comm_fault_pressed)
        self.freq_btn.onclick.do(self.freq_btn_pressed)
        self.volt_btn.onclick.do(self.volt_btn_pressed)
        self.randomize_cbox.onclick.do(self.randomize_pressed)
        self.random_range_btn.onclick.do(self.random_range_pressed)
        #
        self._dmm.set_random_data_state(DEFAULT_RANDOM_STATE)
        #
        return main_box

    def comm_fault_pressed(self, widget) -> None:
        if self.comm_fault_cbox.get_value() is True:
            state = CommunicationState.FAULTED
        else:
            state = CommunicationState.OPENED
        self._dmm.set_communication_state(state)

    def freq_btn_pressed(self, widget) -> None:
        self._dmm.set_frequency(float(self.freq_input.text))

    def volt_btn_pressed(self, widget) -> None:
        self._dmm.set_voltage(float(self.volt_input.text))

    def randomize_pressed(self, widget) -> None:
        if self.randomize_cbox.get_value() is True:
            self._dmm.set_random_data_state(True)
        else:
            self._dmm.set_random_data_state(False)

    def random_range_pressed(self, widget) -> None:
        self._dmm.set_random_range(float(self.random_range_input.text))


# =============================================================================
def check_dmm_emulator()-> None:
    date_fmt = "%d/%m/%Y %H:%M:%S"
    log_format = "%(asctime)s %(levelname) -8s %(filename)s " + \
                 " %(funcName)s (%(lineno)d)-> None: %(message)s"
    self._log.basicConfig(level=self._log.INFO,
                        datefmt=date_fmt,
                        format=log_format)

    dmm = DmmEmulator(pid=0x2a8d, vid=0x1601, timeout=4.8)


if __name__ == '__main__':
    check_dmm_emulator()



# -*- coding: utf-8 -*-

"""package scinstr.daq.labjacq
author    Benoit Dubois
copyright FEMTO Engineering, 2020-2024
licence   GPL 3.0+
brief     Handle Labjack T7(-Pro) device (DAQ board) through USB interface.
details   The USB interface consists of the normal bidirectional control
          endpoint (0 OUT & IN), 3 used bulk endpoints (1 OUT, 2 IN, 3 IN),
          and 1 dummy endpoint (3 OUT). Endpoint 1 consists of a 64 byte OUT
          endpoint (address = 0x01). Endpoint 2 consists of a 64 byte IN
          endpoint (address = 0x82). Endpoint 3 consists of a dummy OUT
          endpoint (address = 0x03) and a 64 byte IN endpoint (address = 0x83).
          Endpoint 3 OUT is not supported, and should never be used.

          All commands should always be sent on Endpoint 1, and the responses
          to commands will always be on Endpoint 2. Endpoint 3 is only used
          to send stream data from the T-series device to the host.
"""

import logging
import time
import usb.core
import usb.util
import pymodbus.exceptions as mdex

# from pymodbus.factory import ClientDecoder
from pymodbus.pdu import DecodePDU as ClientDecoder
from pymodbus.exceptions import ConnectionException

#  pymodbus.framer.FramerSocket module
from pymodbus.framer import FramerSocket as ModbusSocketFramer

# from pymodbus.client.sync import BaseModbusClient
from pymodbus.client.base import ModbusBaseClient
from scinstr.daq.labjack.constants import T7_VID, T7_PID, T7_USB_TIMEOUT
from scinstr.daq.labjack.t7 import T7


DEFAULT_RETRIES_NB = 3

# USB interface parameters
_CONFIG_ID = 1
_INTERFACE_ID = 0  # The interface we use to talk to the device
_ALT_INTF_ID = 0  # The alternate interface use to talk to the device
_OUT_EP_ID = 0  # EP1_OUT @0x01 = 1    -> send command endpoint
_IN_EP_ID = 1  # EP2_IN  @0x82 = 130  -> response endpoint
_OUT_DAT_EP_ID = 2  # EP3_OUT @0x03 = 3    -> !! do not use !!
_IN_DAT_EP_ID = 3  # EP3_IN  @0x83 = 131  -> stream data from T7 to host


# =============================================================================
def get_interfaces():
    """Get available interface of T7 devices.
    Return a list of Usb Device, a named tuple with the following attributes:
    bLength, bDescriptorType, bcdUSB, bDeviceClass, bDeviceSubClass,
    bDeviceProtocol, bMaxPacketSize0, idVendor, idProduct, bcdDevice,
    iManufacturer, iProduct, iSerialNumber, bNumConfigurations, address, bus,
    port_number, port_numbers, speed.
    :returns: list of (usb.core.)Device (list of object)
    """
    interfaces = usb.core.find(idVendor=T7_VID, idProduct=T7_PID, find_all=True)
    return [interface for interface in interfaces]


# =============================================================================
class UsbModbusTcpClient(ModbusBaseClient):
    """'TCP' modbus client through USB connection."""

    def __init__(self, **kwargs):
        self._ep_in = None
        self._ep_out = None
        self._ep_din = None
        self._ep_dout = None
        self._socket = None
        self._vendor_id = kwargs.get("vendor_id", None)
        self._product_id = kwargs.get("product_id", None)
        self.bus = kwargs.get("bus", None)
        self.address = kwargs.get("address", None)
        self.timeout = kwargs.get("timeout", None)
        super().__init__(ModbusSocketFramer(ClientDecoder(is_server=False)))

    def __str__(self):
        """Builds a string representation of the connection.
        Function is used by pymodbus lib.
        !!!!!!
        Seems that usb driver needs to be recognize as UDP client (?Hack?).
        !!!!!!
        :returns: The string representation (str)
        """
        return "UsbModbusUdpClient(%s:%s)" % (self._vendor_id, self._product_id)

    def is_socket_open(self):
        """Check whether the underlying socket is open or not.
        :returns: True if socket/serial is open, False otherwise (bool)
        """
        if self._socket is None:
            return False
        return True

    def close(self):
        """Closes the underlying socket connection"""
        # Use of explicit release because it seems communication with t7 needs
        # deterministic interface release
        usb.util.release_interface(self._socket, _INTERFACE_ID)
        super().close()
        if self._socket is not None:
            del self._socket
            self._socket = None

    def connect(self):
        """Set an object representing our device (usb.core.Device).
        :param vendor_id: identification number of vendor (int)
        :param product_id: identification number of product (int)
        :returns: True if connection is Ok else returns False (bool)
        """
        if self._socket:
            return True
        # Find our device
        if self.bus is not None and self.address is not None:
            self._socket = usb.core.find(
                idVendor=self._vendor_id,
                idProduct=self._product_id,
                bus=self.bus,
                address=self.address,
            )
        else:
            self._socket = usb.core.find(
                idVendor=self._vendor_id, idProduct=self._product_id
            )
        # Was it found?
        if self._socket is None:
            logging.error("Device not found")
            return False
        # Detach the kernel driver if it is active
        if self._socket.is_kernel_driver_active(0):
            try:
                self._socket.detach_kernel_driver(0)
            except usb.core.USBError as ex:
                logging.critical("Could not detach kernel driver: %r", ex)
                return False
        #
        self._config(
            config_id=_CONFIG_ID,
            interface_id=_INTERFACE_ID,
            alt_interface_id=_ALT_INTF_ID,
            in_ep_id=_IN_EP_ID,
            out_ep_id=_OUT_EP_ID,
            in_dat_ep_id=_IN_DAT_EP_ID,
            out_dat_ep_id=_OUT_DAT_EP_ID,
        )
        logging.debug("UsbModbusClient connection OK ")
        return self._socket is not None

    def _config(
        self,
        config_id,
        interface_id,
        alt_interface_id,
        out_ep_id,
        in_ep_id,
        out_dat_ep_id,
        in_dat_ep_id,
    ):
        """Configure T7(-Pro) device and get endpoint instance.
        Endpoint Address description:
        - Bits 0..3 Endpoint Number.
        - Bits 4..6 Reserved. Set to Zero
        - Bits 7 Direction 0 = Out, 1 = In (Ignored for Control Endpoints)
        Example: address = 129 = (81)8 = (1000 0001)2
                 -> Direction: In, Number: 1
        :param config_id: ...
        ...
        :returns: None
        """
        try:
            self._socket.set_configuration(config_id)
        except usb.core.USBError as ex:
            logging.critical("USB error during dev configuration: %r", ex)
            raise
        except Exception as ex:
            logging.critical("Could not set interface configuration: %r", ex)
            raise
        # Set interface
        try:
            self._socket.set_interface_altsetting(
                interface=interface_id, alternate_setting=alt_interface_id
            )
        except usb.core.USBError as ex:
            logging.critical("USB error during interface altsetting: %r", ex)
            raise
        except Exception as ex:
            logging.critical("Error during interface altsetting: %r", ex)
            raise
        # Use of explicit release because it seems communication with t7 needs
        # deterministic interface claiming
        usb.util.claim_interface(self._socket, interface_id)
        # Get endpoint instances
        try:
            self._ep_out = self._socket[config_id - 1][
                (interface_id, alt_interface_id)
            ][out_ep_id]
            self._ep_in = self._socket[config_id - 1][(interface_id, alt_interface_id)][
                in_ep_id
            ]
            self._ep_dout = self._socket[config_id - 1][
                (interface_id, alt_interface_id)
            ][out_dat_ep_id]
            self._ep_din = self._socket[config_id - 1][
                (interface_id, alt_interface_id)
            ][in_dat_ep_id]
        except usb.core.USBError as ex:
            logging.critical("Could not set endpoint setting: %r", ex)
        logging.debug("USB interface succefuly configured")

    def send(self, request, _):
        """Sends data on the underlying socket
        :param request: The encoded request to send
        :return: The number of bytes written
        """
        if not self._socket:
            raise ConnectionException(self.__str__())
        if request:
            try:
                nb = self._ep_out.write(request, self.timeout)
            except usb.core.USBError as ex:
                logging.error("Could not write data: %r", ex)
                return 0
        return nb

    def recv(self, size):
        """Reads data from the underlying descriptor
        :param size: The number of bytes to read
        :return: The bytes read
        """
        if not self._socket:
            raise ConnectionException(self.__str__())

        begin = time.time()
        data = b""

        if size is not None:
            if size < self._ep_din.wMaxPacketSize:
                data = self._ep_in.read(self._ep_din.wMaxPacketSize)
            else:
                while len(data) < size:
                    try:
                        data += self._ep_in.read(self._ep_din.wMaxPacketSize)
                    except usb.core.USBError:
                        pass
                    if not self.timeout or (time.time() - begin > self.timeout):
                        break
        else:
            while True:
                try:
                    data += self._ep_in.read(self._ep_din.wMaxPacketSize)
                except usb.core.USBError:
                    pass
                if not self.timeout or (time.time() - begin > self.timeout):
                    break

        return data


# =============================================================================
class T7Usb(T7):
    """T7Usb class, provide command/response handling of LabJack T7(-Pro)
    board through USB interface.
    """

    def __init__(self, bus=None, address=None):
        super().__init__()
        self.bus = bus
        self.address = address

    def connect(self):
        """Connect to device.
        :returns: None
        """
        logging.info("Connecting to T7")
        try:
            self._client = UsbModbusTcpClient(
                retries=DEFAULT_RETRIES_NB,
                vendor_id=T7_VID,
                product_id=T7_PID,
                bus=self.bus,
                address=self.address,
                timeout=T7_USB_TIMEOUT,
            )
            is_connected = self._client.connect()
        except mdex.ConnectionException as ex:
            logging.error("Modbus connection error: %r", ex)
            return False
        except Exception as ex:
            logging.error("Unexpected exception: %r", ex)
            return False
        if is_connected is False:
            logging.error("Connection failed")
            return False
        logging.info("Connection --> Ok")
        return True

    def close(self):
        """Close connection with device.
        :returns: None
        """
        try:
            self._client.close()
        except Exception as ex:
            logging.error("Unexpected error %r", ex)
        else:
            logging.info("Connection to T7 closed")


# =============================================================================
if __name__ == "__main__":
    # For "Ctrl+C" works
    import signal

    signal.signal(signal.SIGINT, signal.SIG_DFL)

    # Setup logger
    LOG_FORMAT = "%(asctime)s %(levelname)s %(filename)s (%(lineno)d): " + "%(message)s"
    logging.basicConfig(format=LOG_FORMAT, level=logging.INFO)

    I0 = 200e-6

    def pt100_v2t(volt, i0=I0):
        """Temperature from voltage of a Pt100 sensor."""
        return (2.604 * volt / i0) - 260.4

    print("Find T7 device(s) with following bus and address:")
    interfaces = get_interfaces()
    for interface in interfaces:
        print("bus: {}, address: {}".format(interface.bus, interface.address))

    _FILE = "t7.dat"

    T7 = T7Usb()
    if not (T7.connect()):
        print("Connection to T7 failed.")
        exit()

    # AINS_LIST = [0, 1, 2, 3]
    # T7.set_ains_resolution(AINS_LIST, [8, 8, 8, 8])

    T7.enable_high_speed_counter(2)

    with open(_FILE, "a") as fd:
        while True:
            # V_LIST = T7.get_ains_voltage(AINS_LIST)
            # V_STRING = ''
            # for V in V_LIST:
            #    V_STRING += str(V) + ';'
            # print(V_STRING)
            # v0 = v_list[1]-v_list[0]
            # v1 = v_list[2]-v_list[3]
            # t0 = pt100_v2t(v0)
            # t1 = pt100_v2t(v1)
            # print(v0, v1,t0, t1)
            # fd.write(v_string + '\n')
            # fd.flush()

            print("Frequency:", T7.get_high_speed_counter_frequency(2))
            time.sleep(1.0)

    T7.close()
    exit()



# -*- coding: utf-8 -*-

"""package scinstr
author    Benoit Dubois
copyright FEMTO ENGINEERING, 2019
license   GPL v3.0+
brief     API dedicated to handle the 34972a datalogger through ethernet.
"""

import logging
import re
import socket
from collections import namedtuple
from time import sleep

from PyQt5.QtCore import QObject, pyqtSignal, pyqtSlot

from scinstr.daq.constants import DAQ34972A_ID, DAQ34972A_PORT, DAQ34972A_TIMEOUT

DEFAULT_PORT = DAQ34972A_PORT
DEFAULT_TIMEOUT = DAQ34972A_TIMEOUT
DEVICE_ID = DAQ34972A_ID

# Define a new type used for storing device configurations
DTuple = namedtuple('DTuple', ['mnemo', 'caption', 'list'])

# =============================================================================
RESO = [DTuple("0.0001", "0.0001 x range", None),
        DTuple("0.00001", "0.00001 x range", None),
        DTuple("0.000003", "0.000003 x range", None),
        DTuple("0.0000022", "0.0000022 x range", None),
        DTuple("0.000001", "0.000001 x range", None),
        DTuple("0.0000008", "0.0000008 x range", None),
        DTuple("0.0000003", "0.0000003 x range", None),
        DTuple("0.00000022", "0.00000022 x range", None)]

INTGT = [DTuple("0.02", "0.02 PLC", None),
         DTuple("0.2", "0.2 PLC", None),
         DTuple("1", "1 PLC", None),
         DTuple("2", "2 PLC", None),
         DTuple("10", "10 PLC", None),
         DTuple("20", "20 PLC", None),
         DTuple("100", "100 PLC", None),
         DTuple("200", "200 PLC", None)]

RES_RANGE = [DTuple("DEF", "Automatique", None),
             DTuple("100", "100 Ω", None),
             DTuple("1E+3", "1 kΩ", None),
             DTuple("10E+3", "10 kΩ", None),
             DTuple("100E+3", "100 kΩ", None),
             DTuple("1E+6", "1 MΩ", None),
             DTuple("10E+6", "10 MΩ", None),
             DTuple("100E+6", "100 MΩ", None)]

VOLT_RANGE = [DTuple("DEF", "Automatique", None),
              DTuple("0.1", "100 mV", None),
              DTuple("1", "1 V", None),
              DTuple("10", "10 V", None),
              DTuple("100", "100 V", None),
              DTuple("300", "300 V", None)]

FREQ_RANGE = [DTuple("DEF", "Automatique", None),
              DTuple("3", "3 Hz", None),
              DTuple("20", "20 Hz", None),
              DTuple("200", "200 Hz", None)]

PER_RANGE = [DTuple("DEF", "Automatique", None),
             DTuple("1", "1 second", None),
             DTuple("0.1", "100 ms", None),
             DTuple("0.01", "10 ms", None)]

CUR_RANGE = [DTuple("DEF", "Automatique", None),
             DTuple("0.01", "10 mA", None),
             DTuple("0.1", "100 mA", None),
             DTuple("1", "1 A", None)]

TCOUPLE_TYPE = [DTuple("B", "Type B", None),
                DTuple("E", "Type E", None),
                DTuple("J", "Type J", None),
                DTuple("K", "Type K", None),
                DTuple("N", "Type N", None),
                DTuple("R", "Type R", None),
                DTuple("S", "Type S", None),
                DTuple("T", "Type T", None)]

RTD_TYPE = [DTuple("85", "Type 85", None),
            DTuple("91", "Type 91", None)]

THERM_TYPE = [DTuple("2252", "Type 2252", None),
              DTuple("5000", "Type 5000", None),
              DTuple("10000", "Type 10000", None)]

TEMP_TRANS_TYPE = [DTuple("DEF", "Automatique", None),
                   DTuple("TCouple", "Thermocouple", TCOUPLE_TYPE),
                   DTuple("RTD", "2-wire RTD", RTD_TYPE),
                   DTuple("FRTD", "4-wire RTD", RTD_TYPE),
                   DTuple("THER", "Thermistor", THERM_TYPE)]

FUNCTION = [DTuple("VOLT:DC", "DC Voltage", VOLT_RANGE),
            DTuple("VOLT:AC", "AC Voltage", VOLT_RANGE),
            DTuple("FRES", "4-wire Resistance", RES_RANGE),
            DTuple("RES", "2-wire Resistance", RES_RANGE),
            DTuple("FREQ", "Frequency", FREQ_RANGE),
            DTuple("PER", "Period", FREQ_RANGE),
            DTuple("TEMP", "Temperature", TEMP_TRANS_TYPE)]

FUNCTION_FULL = [DTuple("VOLT:DC", "DC Voltage", VOLT_RANGE),
                 DTuple("VOLT:AC", "AC Voltage", VOLT_RANGE),
                 DTuple("FRES", "4-wire Resistance", RES_RANGE),
                 DTuple("RES", "2-wire Resistance", RES_RANGE),
                 DTuple("FREQ", "Frequency", FREQ_RANGE),
                 DTuple("PER", "Period", PER_RANGE),
                 DTuple("TEMP", "Temperature", TEMP_TRANS_TYPE),
                 DTuple("CURR:DC", "DC Current", CUR_RANGE),
                 DTuple("CURR:AC", "AC Current", CUR_RANGE)]


# =============================================================================
class Daq34972aEth(QObject):
    """Class used to handle the ethernet facilities of 34972a device.
    """

    connected = pyqtSignal()
    closed = pyqtSignal()
    id_checked = pyqtSignal(bool)

    def __init__(self, ip="",
                 port=DEFAULT_PORT,
                 timeout=DEFAULT_TIMEOUT):
        """The constructor.
        :param ip: IP address of device (str)
        :param port: Ethernet port of device (int)
        :param timeout: Socket timeout value in second (float)
        :returns: None
        """
        super(Daq34972aEth, self).__init__()
        self.ip = ip
        self.port = port
        self.timeout = timeout
        self._sock = None
        self._errors = []  # List of current errors

    @pyqtSlot()
    def connect(self):
        """Specific ethernet connection process to 34972a.
        :returns: True if connection effective else False (bool)
        """
        try:
            self._sock = socket.socket(socket.AF_INET,
                                       socket.SOCK_STREAM,
                                       socket.IPPROTO_TCP)
            self._sock.settimeout(self.timeout)
            self._sock.connect((self.ip, self.port))
        except ValueError as ex:
            logging.error("Connection parameters out of range: %r", ex)
            return False
        except socket.timeout:
            logging.error("Timeout on connection")
            return False
        except Exception as ex:
            logging.critical("Unexpected exception during connection with "
                             + "34972a: %r", ex)
            return False
        else:
            self.reset()
            self.connected.emit()
            logging.debug("Connected to 34972A")
            return True

    @pyqtSlot()
    def close(self):
        """Specific ethernet closing process with 34972A.
        :returns: None
        """
        try:
            self._sock.close()
        except socket.error as ex:
            logging.error("Socket error %r", ex)
        except Exception as ex:
            logging.error("Unexpected error %r", ex)
        else:
            self._sock = None
            self.closed.emit()
            logging.info("Connection to 34972A closed")

    @pyqtSlot()
    def check_interface(self):
        """Returns True if interface with device is OK.
        :returns: status of interface with device (boolean)
        """
        if self.connect() is False:
            self.id_checked.emit(False)
            return False
        try:
            _id = self.get_id()
        except Exception:  # Catch connection and  get_id problem
            _id = ""  # To avoid error with 'find()' when '_id' is not defined
        if _id.find(DEVICE_ID) >= 0:
            self.id_checked.emit(True)
            return True
        else:
            self.id_checked.emit(False)
            return False

    @pyqtSlot()
    def reset(self):
        """Resets acquisition process and device.
        :returns: None
        """
        self._sock.send("*RST\n")
        self._sock.send("*CLS\n")
        logging.debug("Reset 34972A device")

    def local_mode(self):
        """Set device in local mode.
        :returns: None
        """
        self.write("SYST:LOC")

    def set_timeout(self, timeout):
        """Set timeout on socket operations.
        :param timeout: timeout value in second (float)
        :returns: None
        """
        self._sock.settimeout(timeout)
        self.timeout = timeout

    def get_timeout(self):
        """Get timeout on socket operations.
        :returns: timeout value in second (float)
        """
        return self._sock.gettimeout()

    def set_ip(self, ip):
        """Set IP address used to speak with device.
        :param ip: IP address (str)
        :return: None
        """
        self.ip = ip

    def get_ip(self):
        """Get IP used to speak with device.
        :returns: IP address (str)
        """
        return self.ip

    def set_port(self, port):
        """Set internet port used to speak with device.
        :param port: port used by dmm4050 (int)
        :returns: None
        """
        self.port = port

    def get_port(self):
        """Get internet port used to speak with device.
        :returns: port used by dmm4050 (int)
        """
        return self.port

    def get_id(self):
        """Get ID of device.
        :returns: ID of device (str)
        """
        self.write("*IDN?")
        return self.read(100)

    def get_error(self):
        """"Return list of current error.
        :returns: list of error (list of str)
        """
        errors = [self.query("SYST:ERR?", 100)]
        while errors[-1] != "+0,\"No error\"":
            errors.append(self.query("SYST:ERR?", 100))
        return errors

    def write(self, data):
        """"Ethernet writing process.
        :param data: data writes to device (str)
        :returns: None
        """
        try:
            self._sock.send(data + "\n")
        except socket.timeout:
            logging.error("Timeout")
        except Exception as ex:
            logging.error(str(ex))
        logging.debug("write " + data.strip('\n'))

    def read(self, length):
        """Ethernet reading process.
        :param length: length of message to read (int)
        :returns: Message reads from device (str)
        """
        try:
            retval = self._sock.recv(length)
        except socket.timeout:
            logging.error("Timeout")
        except Exception as ex:
            logging.error(str(ex))
        else:
            logging.debug("read: " + retval.strip('\n'))
            return retval

    def query(self, data, length=100):
        """Read after write method.
        :param data: data writes to device (str)
        :param length: length of message to read (int)
        :returns: Message returned by device (str)
        """
        self.write(data)
        return self.read(length)


# =============================================================================
class Daq34972a(Daq34972aEth):
    """Class used to handle the 34972a device through ethernet connection
    (inherits of 34972aEth class).
    """

    new_data = pyqtSignal((list,), (str,))

    def __init__(self, ip, port, timeout):
        """The constructor.
        :param ip: IP address of device (str)
        :param port: Ethernet port of device (int)
        :param timeout: Socket timeout value in second (float)
        :returns: None
        """
        super(Daq34972a, self).__init__(ip, port, timeout)

    def get_scanlist(self):
        """Gets scan list from real device.
        :returns: a list of scan (list of str)
        """
        self.write("ROUT:SCAN?")
        retval = self.read(1000)
        retval = retval.strip('\n').split('@')[1][:-1].split(',')
        if retval == ['']:
            return retval
        scanlist = []
        for val in retval:
            if 99 < int(val) < 399:
                scanlist.append(val)
            else:
                logging.error("Bad scan value: " + val)
                raise RuntimeError("Bad scan value %r", val)
        return scanlist

    def data_read(self):
        """Takes a measurement the next time the trigger condition is met.
        After the measurement is taken, the reading is placed in the output
        buffer. "data_read" will not cause readings to be stored in the Meter’s
        internal memory.
        :returns: data read in buffer (list of float)
        """
        try:
            self.write("READ?")
            retval = self.read(1000)
            if re.match('[\+\-]\d\.\d*E[\+\-]\d\d', retval) is None:
                return None
        except Exception as ex:
            logging.error(str(ex))
        else:
            self.new_data[str].emit(retval)
            float_list = [float(value) for value in retval.split(',')]
            self.new_data[list].emit(float_list)
            logging.debug("Data read measurement: " + retval.strip('\n'))
            return float_list

    def _wait_dataready(self, nbread):
        """Waits until 'nbread' readings are stored in memory.
        :param nbread: number of readings triggering end of wait (int)
        :returns: None
        """
        count = 0
        while count < nbread:
            sleep(0.1)
            self.write("DATA:POIN?")
            count = int(self.read(10))


# =============================================================================
def check_34972a():
    """Checks the 34972a class: connects to the data logger, configures a dc
    voltage measurement on channel 101 to 105 then collects and prints data to
    the standard output.
    """

    # For "Ctrl+C" works
    import signal
    signal.signal(signal.SIGINT, signal.SIG_DFL)

    # Specific function imports
    import sys
    from datetime import datetime
    # from mjdutil import datetime_to_mjd
    from PyQt5.QtCore import QCoreApplication

    # Handles log
    date_fmt = "%d/%m/%Y %H:%M:%S"
    log_format = "%(asctime)s %(levelname) -8s %(filename)s " + \
                 " %(funcName)s (%(lineno)d): %(message)s"
    logging.basicConfig(level=logging.INFO,
                        datefmt=date_fmt,
                        format=log_format)

    def print_data(data):
        """'Virtual slot', used to print data emited by a signal.
        Adds also a timetag.
        :returns: None
        """
        # now = datetime_to_mjd(datetime.utcnow())
        now = datetime.utcnow()
        print(str(now) + "," + data)

    # Creates the application
    app = QCoreApplication(sys.argv)
    sys.exit(app.exec_())


# =============================================================================
if __name__ == '__main__':
    check_34972a()



# -*- coding: utf-8 -*-

"""package scinstr.vacuum
author    Benoit Dubois
copyright FEMTO ENGINEERING, 2018-2024
license   GPL v3.0+
brief     Base class to handle vacuum gauge device.
"""

import logging
import socket
import serial


# Control Symbols
ACK = '\x06' # AcKnowledge
NAK = '\x15' # Not AcKnowledge
ENQ = '\x05' # Enquiry (Request for data transmission)
ETX = '\x03' # End of text (Ctrl-C)
CR = '\x0D'
LF = '\x0A'

LINE_TERMINATION = CR + LF  # CR, LF and CRLF

# Generic mnemonics
MNEMONIQUE = [
    'BAU', # Baud rate
    'CAx', # Calibration factor Sensor x
    'CID', # Measurement point names
    'DCB', # Display control Bargraph
    'DCC', # Display control Contrast
    'DCD', # Display control Digits
    'DCS', # Display control Screensave
    'DGS', # Degas
    'ERR', # Error Status
    'FIL', # Filter time constant
    'FSR', # Full scale range of linear sensors
    'LOC', # Parameter setup lock
    'NAD', # Node (device) address for RS485
    'OFC', # Offset correction
    'OFC', # Offset correction
    'PNR', # Program number
    'PRx', # Status, Pressure sensor x (1 ... 6)
    'PUC', # Underrange Ctrl
    'RSX', # Interface
    'SAV', # Save default
    'SCx', # Sensor control
    'SEN', # Sensor on/off
    'SPx', # Set Point Control Source for Relay x
    'SPS', # Set Point Status A,B,C,D,E,F
    'TAI', # Test program A/D Identify
    'TAS', # Test program A/D Sensor
    'TDI', # Display test
    'TEE', # EEPROM test
    'TEP', # EPROM test
    'TID', # Sensor identification
    'TKB', # Keyboard test
    'TRA', # RAM test
    'UNI', # Unit of measurement (Display)
    'WDT' # Watchdog and System Error Control
]

ES = {
    "0000": "No error",
    "1000": "Error", # Controller error (See display on front panel)
    "0100": "NO HWR", # No hardware
    "0010": "PAR", # Inadmissible parameter
    "0001": "SYN" # Syntax error
}


# =============================================================================
class Gauge():
    """Base class to handle vacuum controller device.
    """

    def __init__(self, termination=LINE_TERMINATION):
        self._termination = termination

    def connect(self):
        """Connect to the remote host
        :returns: True if connection succeeded, False otherwise
        """
        raise NotImplementedError("Method not implemented by derived class")

    def close(self):
        """ Closes the underlying connection
        """
        raise NotImplementedError("Method not implemented by derived class")

    def _write(self, data):
        """Writing process.
        :param data: data writes to device (str)
        :returns: number of bytes sent (int)
        """
        raise NotImplementedError("Method not implemented by derived class")

    def _read(self, length):
        """Reading process.
        :param length: length of message to read (int)
        :returns: Message reads from device (str)
        """
        raise NotImplementedError("Method not implemented by derived class")

    def read(self, length=128):
        """Basic read process.
        :param length: max number of bytes to read, None -> no limit (int)
        :returns: data received or False if communication error (str)
        """
        result = self._read(length).strip(self._termination)
        logging.debug("read %r", result)
        return result

    def write(self, mnemo, parameters=None):
        """Write message to device.
        :param mnemo: mnemonique (command/response) (str)
        :param parameters: parameters if needed (str)
        :returns: True if transmission OK else False (bool)
        """
        if parameters is not None:
            msg = mnemo + ',' + parameters + CR
        else:
            msg = mnemo + CR
        self._write(msg)
        is_ack = self._read(3)
        if is_ack == (ACK + CR + LF):
            logging.debug("write %r acknoledged", msg.strip(CR))
            return True
        logging.debug("write %r non acknoledged", msg.strip(CR))
        return False

    def query(self, mnemo, parameters=None, size=100):
        """Ask procedure ie read data after write.
        :param mnemo: mnemonique (command/response) (str)
        :param parameters: parameters if needed (str)
        :returns: data received (str)
        """
        write_ok = self.write(mnemo, parameters)
        if write_ok is False:
            logging.error("Write to device failed")
            return ''
        self._write(ENQ)
        result = self.read(size).strip(LINE_TERMINATION)
        return result

    def reset(self, cancel_error=False):
        """Set the instrument functions to the factory default power up state.
        Return all present error.
        'No error': 0,
        'Watchdog has responded': 1,
        'Task fail error': 2,
        'EPROM error': 3,
        'RAM error': 4,
        'EEPROM error': 5,
        'DISPLAY error': 6,
        'A/D converter error': 6,
        'Gauge 1 error (e.g. filament rupture, no supply)': 7,
        'Gauge 1 identification error': 8,
        'Gauge 2 error (e.g. filament rupture, no supply)': 8,
        'Gauge 2 identification error': 9
        """
        if cancel_error is True:
            param = '1'
        else:
            param = None
        return self.query('RES', param)

    @property
    def idn(self):
        """Gauge identification.
        """
        return self.query('TID')

    @property
    def degas(self):
        """Return degas gauges status.
        """
        degas_status = self.query("DGS").split(',')
        return degas_status

    @degas.setter
    def degas(self, channel, value=1):
        """Degas gauge. Return degas gauge status.
        """
        if channel == 1:
            param = str(value)
        elif channel == 2:
            param = ',' + str(value)
        else:
            logging.error(f"Bad channel number selection for degas: {channel}")
            return
        degas_status = self.query("DGS", param).split(',')[channel-1]
        return degas_status

    @property
    def display_channel(self):
        """Set measurement channel to be displayed.
        Return displayed channel.
        """
        return self.query('SCT')

    @display_channel.setter
    def display_channel(self, channel):
        """Set measurement channel to be displayed.
        Return displayed channel.
        """
        return self.query('SCT', channel-1)

    @property
    def error(self):
        """Return error status.
        """
        return self.query('ERR')

    @property
    def gauge_enabled(self):
        """Return gauge status.
        """
        gauges_status = self.query('SEN').split(',')
        return gauges_status

    @gauge_enabled.setter
    def gauge_enabled(self, channel, value):
        """Set gauge on/off. Return gauge status.
        """
        if channel == 1:
            param = str(value)
        else:
            param = ',' + str(value)
        gauge_status = self.query('SEN', param).split(',')[channel-1]
        return gauge_status

    def pressure(self, channel):
        """Pressure measurement gauge 1 or 2.
        Return status of measurement and measurement data.
        """
        ans = self.query('PR{:d}'.format(channel)).split(',')
        pressure = float(ans[1])
        status = ans[0]
        return status, pressure

    def get_filter_cst(self, channel):
        """Get time constant of measurement filters.
        :returns: current filters time constant (int or list of int)
        """
        time_cst = self.query("FIL").split(',')[channel-1]
        return time_cst

    def set_filter_cst(self, channel, value):
        """Set time constant of measurement filters.
        0 –> fast
        1 –> medium (default)
        2 –> slow
        Return current filter time constant.
        """
        if channel == 1:
            param = str(value)
        else:
            param = ',' + str(value)
        time_cst = self.query("FIL", param).split(',')[channel-1]
        return time_cst

    @property
    def unit(self):
        """Get control unit.
        """
        return self.query('UNI')

    @unit.setter
    def unit(self, value):
        """Set control unit.
        0 –> mbar/bar (default)
        1 –> Torr
        2 –> Pascal
        """
        return self.query('UNI', value)

    @property
    def baud(self):
        """Transmission rate.
        """
        return self.query('BAU')

    @baud.setter
    def baud(self, value):
        """Transmission rate.
        """
        return self.query('BAU', value)

    @property
    def digit_display(self):
        """Display resolution.
        """
        return self.query('DCD')

    @digit_display.setter
    def digit_display(self, value):
        """Display resolution.
        """
        return self.query('DCD', value)

    def get_full_scale_range(self, channel):
        """Return full scale range value of the measurement range of
        linear gauges (int or list of int).
        """
        scale_range = self.query('FSR').split(',')[channel-1]
        return scale_range

    def set_full_scale_range(self, channel, value):
        """Set full scale value of the measurement range of linear gauges.
        """
        if channel == 1:
            param = str(value)
        else:
            param = ',' + str(value)
        scale_range = self.query("FSR", param).split(',')[channel-1]
        return scale_range

    def get_calibration(self, channel):
        """Return calibration factor of specified gauge.
        """
        calibration_factor = self.query('CAL').split(',')[channel-1]
        return calibration_factor

    def set_calibration(self, channel, value):
        """Set calibration factor.
        """
        if channel == 1:
            param = str(value)
        else:
            param = ',' + str(value)
        calibration_factor = self.query("CAL", param).split(',')[channel-1]
        return calibration_factor

    def get_offset_correction(self, channel):
        """Return offset correction factor of specified gauge.
        """
        ofc = self.query('OFC').split(',')[channel-1]
        return ofc

    def set_offset_correction(self, channel, value):
        """Set offset correction for linear gauge.
        """
        if channel == 1:
            param = str(value)
        else:
            param = ',' + str(value)
        ofc = self.query("OFC", param).split(',')[channel-1]
        return ofc

    def get_offset_display(self, channel):
        """Return offset display factor of specified gauge.
        """
        ofd = self.query('OFD').split(',')[channel-1]
        return ofd

    def set_offset_display(self, channel, value):
        """Set offset display for linear gauge.
        """
        if channel == 1:
            param = str(value)
        else:
            param = ',' + str(value)
        ofd = self.query("OFD", param).split(',')[channel-1]
        return ofd

    def get_switching_threshold(self, key):
        """Get threshold value setting allocation.
        Return lower and upper threshold value.
        """
        pass

    def set_switching_threshold(self, channel, function,
                            lower_threshold, upper_threshold):
        """Set threshold value setting allocation.
        Return lower and upper threshold value.
        """
        param = str(channel-1) + ',' \
                + str(lower_threshold) + ',' \
                + str(upper_threshold)
        func, lth, uth = self.query("SP{:d}".format(function), param).split(',')
        return func, lth, uth

    def get_switching_function_status(self):
        """Return switching function status
        """
        function_status = self.query('SPS').split(',')
        return function_status


# =============================================================================
class GaugeEth(Gauge):
    """Handle vacuum controller device through ethernet interface.
    """

    def __init__(self, ip, port, timeout):
        super().__init__()
        self._sock = None
        self.ip = ip
        self.port = port
        self.timeout = timeout

    def connect(self):
        """Open connection with device.
        :param ip: IP address of device (str)
        :param port: Device port in use (int)
        :param timeout: Timeout in second (float)
        :returns: True if connection succeed False elsewhere (bool)
        """
        self._sock = socket.socket(socket.AF_INET,
                                   socket.SOCK_STREAM,
                                   socket.IPPROTO_TCP)
        self._sock.settimeout(self.timeout)
        try:
            self._sock.connect((self.ip, self.port))
        except ValueError as ex:
            logging.error("Wrong connection parameters: %r", ex)
            return False
        except socket.timeout:
            logging.error("Timeout on connection")
            return False
        logging.info("Connected to gauge")
        return True

    def close(self):
        """Close connection with device.
        :returns: None
        """
        if self._sock is None:
            return
        self._sock.close()
        self._sock = None
        logging.info("Connection to gauge closed")

    def _write(self, data):
        """Specific ethernet writing process.
        :param data: data writes to device (str)
        :returns: number of bytes sent (int)
        """
        try:
            nb_bytes = self._sock.send(data.encode('utf-8'))
        except socket.timeout:
            logging.error("Device write timeout")
            return 0
        return nb_bytes

    def _read(self, length):
        """Specific ethernet reading process.
        :param length: length of message to read (int)
        :returns: Message reads from device (str)
        """
        try:
            retval = self._sock.recv(length).decode('utf-8')
        except socket.timeout:
            logging.error("Device read timeout")
            return ''
        return retval


# =============================================================================
class GaugeSerial(Gauge):
    """Handle vacuum controller device through USB connection.
    The USB interface emulates a 'classic' serial interface.
    """

    def __init__(self, port=None, baudrate=None, bytesize=None,
                 parity=None, stopbits=None, timeout=1.0):
        super().__init__()
        self._ser = serial.Serial()
        self.port = port
        self.baudrate = baudrate
        self.bytesize = bytesize
        self.parity = parity
        self.stopbits = stopbits
        self.timeout = timeout

    def connect(self):
        """Connect to the remote host
        :returns: True if connection succeeded, False otherwise
        """
        if self._ser.isOpen() is False:
            try:
                self._ser.open()
            except ValueError as ex:
                logging.error("Wrong connection parameters: %r", ex)
                return False
            except serial.SerialTimeoutException:
                logging.error("Timeout on connection")
                return False
            except serial.serialutil.SerialException as ex:
                logging.error("Serial error: %r", ex)
                return False
        logging.info("Connected to gauge")
        self._ser.flush()
        return True

    def close(self):
        """Closes the underlying serial connection
        """
        if self._ser is not None:
            try:
                self._ser.close()
            except Exception as ex:
                logging.error("Error when closing USB connection: %r", ex)
        self._ser = None
        logging.info("Connection to gauge closed")

    def _write(self, data):
        """Specific USB writing process.
        :param data: data writes to device (str)
        :returns: number of bytes sent (int)
        """
        try:
            nb_bytes = self._ser.write(data.encode('utf-8'))
        except serial.SerialTimeoutException:
            logging.error("Device write timeout")
            return 0
        return nb_bytes

    def _read(self, length):
        """Specific USB reading process.
        :param length: length of message to read (int)
        :returns: Message reads from device (str)
        """
        try:
            retval = self._ser.read(length).decode('utf-8')
        except serial.SerialTimeoutException:
            logging.error("Device read timeout")
            return ''
        return retval

    @property
    def port(self):
        return self._ser.port

    @port.setter
    def port(self, value):
        self._ser.port = value

    @property
    def baudrate(self):
        return self._ser.baudrate

    @baudrate.setter
    def baudrate(self, value):
        self._ser.baudrate = value

    @property
    def bytesize(self):
        return self._ser.bytesize

    @bytesize.setter
    def bytesize(self, value):
        self._ser.bytesize = value

    @property
    def parity(self):
        return self._ser.parity

    @parity.setter
    def parity(self, value):
        self._ser.parity = value

    @property
    def stopbits(self):
        return self._ser.stopbits

    @stopbits.setter
    def stopbits(self, value):
        self._ser.stopbits = value

    @property
    def timeout(self):
        return self._ser.timeout

    @timeout.setter
    def timeout(self, value):
        self._ser.timeout = value



# -*- coding: utf-8 -*-

# Default Tektronix DMM 4050 parameters
DMM4050_ID = "TEKTRONIX,DMM4050"
DMM4050_PORT = 3490
DMM4050_TIMEOUT = 3.0

# Default Keysight DMM 3446(0|1)A parameters
DMM34461A_ID = "Technologies,3446"
DMM34461A_PORT = 5025
DMM34461A_TIMEOUT = 3.0
DMM34461A_VID = 0x2a8d
DMM34461A_PID = 0x1601

# Default Siglent SDM3065X parameters
SDM3065X_ID = "Siglent Technologies,SDM30"
SDM3065X_PORT = 5025
SDM3065X_TIMEOUT = 3.0
SDM3065X_VID = 0xf4ec
SDM3065X_PID = 0xee38

# Default Siglent SDM3055 parameters
SDM3055_ID = "Siglent Technologies,SDM30"
SDM3055_PORT = 5025
SDM3055_TIMEOUT = 3.0
SDM3055_VID = 0xf4ec
SDM3055_PID = 0x1201



# -*- coding: utf-8 -*-

"""package scinstr
author    Benoit Dubois
copyright FEMTO ENGINEERING, 2019
license   GPL v3.0+
brief     API dedicated to handle the Keysight 34461A digital multimeter
          with PyQt signal/slot capabilities.
"""

import logging
from PyQt5.QtCore import QObject, pyqtSignal, pyqtSlot

from scinstr.dmm.dmm34461a import Dmm34461aUsb, Dmm34461aEth
import scinstr.dmm.constants as scst


# =============================================================================
class QObjectAdapter(QObject):
    """cf https://rhettinger.wordpress.com/2011/05/26/super-considered-super/
    "How to Incorporate a Non-cooperative Class"
    """
    def __init__(self, **kwargs):
        self._object = QObject(*kwargs)
        super().__init__(**kwargs)


# =============================================================================
class QDmm34461aUsb(QObjectAdapter, Dmm34461aUsb):
    """Class derived from Dmm34461aEth class to add PyQt facilities.
    """

    connected = pyqtSignal()
    closed = pyqtSignal()
    id_checked = pyqtSignal(bool)
    out_updated = pyqtSignal((float,), (str,))

    def __init__(self, vendor_id=scst.DMM34461A_VID,
                 product_id=scst.DMM34461A_PID,
                 timeout=scst.DMM34461A_TIMEOUT):
        QObject.__init__(self)
        Dmm34461aUsb.__init__(self, vendor_id, product_id, timeout)

    @pyqtSlot(object)
    def connect(self, **kwargs):
        """Abstract protocol connect process. Derived classes must implement
        the connect process dedicated to the specific protocol used.
        :returns: None
        """
        retval = Dmm34461aUsb.connect(self, **kwargs)
        if retval is True:
            self.connected.emit()
        return retval

    @pyqtSlot()
    def close(self):
        """Abstract protocol closing process. Derived classes must implement
        the closing process dedicated to the specific protocol used.
        :returns: None
        """
        Dmm34461aUsb.connect(self)
        self.closed.emit()

    @pyqtSlot()
    def check_interface(self):
        retval = Dmm34461aUsb.check_interface(self)
        self.id_checked.emit(retval)
        return retval

    @pyqtSlot()
    def data_read(self):
        retval, errors = Dmm34461aUsb.data_read(self)
        if retval is not None:
            self.out_updated[str].emit(str(retval))
            self.out_updated[float].emit(retval)
            return retval, errors

    @pyqtSlot(float)
    def set_timeout(self, timeout):
        """Sets timeout on operations.
        :param timeout: timeout value in second (float)
        :returns: None
        """
        self.timeout = timeout

    @pyqtSlot()
    def get_timeout(self):
        """Gets timeout on socket operations.
        :returns: timeout value in second (float)
        """
        return self.timeout

    @pyqtSlot(str)
    def set_pid(self, pid):
        """
        :param pid:
        :returns: None
        """
        self.pid = pid

    @pyqtSlot()
    def get_pid(self):
        """
        :returns: pid
        """
        return self.pid

    @pyqtSlot(str)
    def set_vid(self, vid):
        """
        :param vid:
        :returns: None
        """
        self.vid = vid

    @pyqtSlot()
    def get_vid(self):
        """
        :returns: vid
        """
        return self.vid


# =============================================================================
class QDmm34461aEth(QObjectAdapter, Dmm34461aEth):
    """Class derived from Dmm34461aEth class to add PyQt facilities.
    """

    connected = pyqtSignal()
    closed = pyqtSignal()
    id_checked = pyqtSignal(bool)
    out_updated = pyqtSignal((float,), (str,))

    def __init__(self, ip=None, port=scst.DMM34461A_PORT,
                 timeout=scst.DMM34461A_TIMEOUT):
        QObject.__init__(self)
        Dmm34461aEth.__init__(self, ip, port, timeout)

    @pyqtSlot(object)
    def connect(self, **kwargs):
        """Abstract protocol connect process. Derived classes must implement
        the connect process dedicated to the specific protocol used.
        :returns: None
        """
        retval = Dmm34461aEth.connect(self, **kwargs)
        if retval is True:
            self.connected.emit()
        return retval

    @pyqtSlot()
    def close(self):
        """Abstract protocol closing process. Derived classes must implement
        the closing process dedicated to the specific protocol used.
        :returns: None
        """
        Dmm34461aEth.connect(self)
        self.closed.emit()

    @pyqtSlot()
    def check_interface(self):
        retval = Dmm34461aEth.check_interface(self)
        self.id_checked.emit(retval)
        return retval

    @pyqtSlot()
    def data_read(self):
        retval = Dmm34461aEth.data_read(self)
        if retval is not None:
            self.out_updated[str].emit(str(retval))
            self.out_updated[float].emit(retval)
            return retval

    @pyqtSlot(float)
    def set_timeout(self, timeout):
        """Sets timeout on socket operations.
        :param timeout: timeout value in second (float)
        :returns: None
        """
        self._sock.settimeout(timeout)

    @pyqtSlot()
    def get_timeout(self):
        """Gets timeout on socket operations.
        :returns: timeout value in second (float)
        """
        return self._sock.gettimeout()

    @pyqtSlot(str)
    def set_ip(self, ip):
        """Sets IP address used to speak with device.
        :param ip: IP address (str)
        :return: None
        """
        self._ip = ip

    @pyqtSlot()
    def get_ip(self):
        """Gets IP used to speak with device.
        :returns: IP address (str)
        """
        return self._ip

    @pyqtSlot(int)
    def set_port(self, port):
        """Sets internet port used to speak with device.
        :param port: port used by counter532x0a (int)
        :returns: None
        """
        self._port = port

    @pyqtSlot()
    def get_port(self):
        """Gets internet port used to speak with device.
        :returns: port used by counter532x0a (int)
        """
        return self._port


# =============================================================================
def check_qdmm34461a():
    """Check the Dmm34461axx class: connect to the multimeter, configure a dc
    voltage measurement then collect and print data to standard output.
    Values that work:
    - 1PLC with 0.5 de rtimeout
    - 10PLC with 0.8 de rtimeout
    """
    from datetime import datetime
    # from mjdutil import datetime_to_mjd

    date_fmt = "%d/%m/%Y %H:%M:%S"
    log_format = "%(asctime)s %(levelname) -8s %(filename)s " + \
                 " %(funcName)s (%(lineno)d): %(message)s"
    logging.basicConfig(level=logging.INFO,
                        datefmt=date_fmt,
                        format=log_format)

    # dmm34461a = Dmm34461aSerial(serport="/dev/ttyUSB0", timeout=0.8)
    dmm34461a = QDmm34461aUsb(0x2a8d, 0x1601, timeout=4.8)
    # dmm34461a = Dmm34461aEth(ip="192.168.0.61", port=5025, timeout=2.8)
    if dmm34461a.connect() is not True:
        print("Connection failed")
        return
    dmm34461a.reset()
    dmm34461a.write("CONF:VOLT:DC AUTO")  # Autorange
    dmm34461a.write("VOLT:ZERO:AUTO ON")  # Autozero off
    dmm34461a.write("VOLT:DC:NPLC 100")
    # dmm34461a.write("TRIG:SOUR IMM")  # Immediate trigger source
    # dmm34461a.write("TRIG:DEL 0")   # No trigger delay
    # dmm34461a.write("TRIG:COUN 1")  # 1 measurement per trigger
    # dmm34461a.write("SAMP:COUN 1")  # 1 sample per trigger
    try:
        while True:
            value = dmm34461a.data_read()
            now = datetime.utcnow()
            if value is None or value == "":
                print("# No data @", now)
            else:
                # print(now, '{:0.14f}'.format(value.strip('\n')))
                print(now, value)
    except KeyboardInterrupt:
        pass
    except Exception as er:
        print("# Exception during acquisition: " + str(er))
    dmm34461a.reset()
    dmm34461a.write("SYST:LOC\n")
    dmm34461a.close()


# =============================================================================
if __name__ == '__main__':
    check_qdmm34461a()



# -*- coding: utf-8 -*-

"""package stanford
author    Benoit Dubois
copyright FEMTO ENGINEERING, 2018
license   GPL v3.0+
brief     Base class for handling SR8x0 device
"""

import time
from collections import OrderedDict


SENSITIVITY = OrderedDict([
    ('2 nV/fA', 0),
    ('5 nV/fA', 1),
    ('10 nV/fA', 2),
    ('20 nV/fA', 3),
    ('50 nV/fA', 4),
    ('100 nV/fA', 5),
    ('200 nV/fA', 6),
    ('500 nV/fA', 7),
    ('1 uV/pA', 8),
    ('2 uV/pA', 9),
    ('5 uV/pA', 10),
    ('10 uV/pA', 11),
    ('20 uV/pA', 12),
    ('50 uV/pA', 13),
    ('100 uV/pA', 14),
    ('200 uV/pA', 15),
    ('500 uV/pA', 16),
    ('1 mV/nA', 17),
    ('2 mV/nA', 18),
    ('5 mV/nA', 19),
    ('10 mV/nA', 20),
    ('20 mV/nA', 21),
    ('50 mV/nA', 22),
    ('100 mV/nA', 23),
    ('200 mV/nA', 24),
    ('500 mV/nA', 25),
    ('1 V/uA', 26)
])

TCONSTANTS = OrderedDict([
    ('10 us', 0),
    ('30 us', 1),
    ('100 us', 2),
    ('300 us', 3),
    ('1 ms', 4),
    ('3 ms', 5),
    ('10 ms', 6),
    ('30 ms', 7),
    ('100 ms', 8),
    ('300 ms', 9),
    ('1 s', 10),
    ('3 s', 11),
    ('10 s', 12),
    ('30 s', 13),
    ('100 s', 14),
    ('300 s', 15),
    ('1 ks', 16),
    ('3 ks', 17),
    ('10 ks', 18),
    ('30 ks', 19)
])

SAMPLE_RATES = OrderedDict([
    ('62.5 mHz', 0),
    ('125 mHz', 1),
    ('250 mHz', 2),
    ('500 mHz', 3),
    ('1 Hz', 4),
    ('2 Hz', 5),
    ('4 Hz', 6),
    ('8 Hz', 7),
    ('16 Hz', 8),
    ('32 Hz', 9),
    ('64 Hz', 10),
    ('128 Hz', 11),
    ('256 Hz', 12),
    ('512 Hz', 13),
    ('trigger', 14)
])

REFERENCE_TRIGGER = OrderedDict([
    ('zero_crossing', 0),
    ('rising_edge', 1),
    ('faling_edge', 2)
])

INPUT_SHIELD = OrderedDict([
    ('float', 0),
    ('ground', 1)
])

INPUT_COUPLING = OrderedDict([
    ('AC', 0),
    ('DC', 1)
])

INPUT_FILTER = OrderedDict([
    ((False, False), 0),
    ((True, False), 1),
    ((False, True), 2),
    ((True, True), 3)
])

RESERVE_MODE = OrderedDict([
    ('high', 0),
    ('normal', 1),
    ('low', 2)
])

FILTER_SLOPE = OrderedDict([
    ('6 dB/oct', 0),
    ('12 dB/oct', 1),
    ('18 dB/oct', 2),
    ('24 dB/oct', 3)
])

FRONT_OUTPUT = OrderedDict([
    ('display', 0),
    ('X', 1)
])

AUTO_OFFSET_ASYNC_CH = OrderedDict([
    ('x', 1),
    ('y', 2),
    ('r', 3)
])

FAST_MODE = OrderedDict([
    ('Off', 0),
    ('On (Dos)', 1),
    ('On (Windows)', 2)
])


# =============================================================================
class Sr8x0(object):
    """Base class for Stanford Research System DSP Lock-In Amplifier
    model SR8(3|1)0.
    """

    def connect(self):
        """Connect to the remote host
        :returns: True if connection succeeded, False otherwise
        """
        raise NotImplementedError("Method not implemented by derived class")

    def close(self):
        """ Closes the underlying socket connection
        """
        raise NotImplementedError("Method not implemented by derived class")

    def write(self, data):
        """Writing process.
        :param data: data writes to device (str)
        :returns: number of bytes sent (int)
        """
        raise NotImplementedError("Method not implemented by derived class")

    def read(self, length):
        """Reading process.
        :param length: length of message to read (int)
        :returns: Message reads from device (str)
        """
        raise NotImplementedError("Method not implemented by derived class")

    def query(self, data, length=100):
        """Basic query i.e. write then read.
        """
        self.write(data)
        return self.read(length).strip('\n')

    @property
    def idn(self):
        """Instrument identification.
        """
        return self.query('*IDN?')

    def reset(self):
        """Reset GPIB interface of Stanford device
        """
        self.write("*RST")

    def status_byte(self, bit=None):
        """Serial poll status byte, a number between 0-255.
        User can request a specific bit value.
        """
        if bit is None:
            return int(self.query('*STB?'))
        else:
            return int(self.query('*STB? {:d}'.format(bit)))

    def wait_bit0(self):
        """Check the scan in progress bit in the Serial Poll Status
        Byte (bit 0) to determine when a scan is finished.
        """
        bit0 = self.status_byte(0)
        while bit0 == 0:
            time.sleep(1.0)
            bit0 = self.status_byte(0)

    def wait_bit1(self):
        """Check the command execution in progress bit in the Serial Poll
        Status Byte (bit 1) to determine when the function is finished.
        """
        bit1 = self.status_byte(1)
        while bit1 == 0:
            time.sleep(0.5)
            bit1 = self.status_byte(1)

    @property
    def reference_phase_shift(self):
        """Getter of the phase shift of the reference.
        """
        return self.query('PHAS?')

    @reference_phase_shift.setter
    def reference_phase_shift(self, value):
        """Setter of the phase shift of the reference.
        """
        self.write('PHAS{:.2f}'.format(value))

    @property
    def reference_internal(self):
        """Getter of the reference source.
        """
        return self.query('FMOD?')

    @reference_internal.setter
    def reference_internal(self, value):
        """Setter of the reference source.
        """
        self.write('FMOD {}'.format(value))

    @property
    def frequency(self):
        """Getter of the reference frequency.
        """
        return self.query('FREQ?')

    @frequency.setter
    def frequency(self, value):
        """Setter of the reference frequency.
        """
        self.write('FREQ {:.5f}'.format(value))

    @property
    def reference_trigger(self):
        """Getter of the reference trigger when using the external
        reference mode.
        """
        return self.query('RSLP?')

    @reference_trigger.setter
    def reference_trigger(self, value):
        """Setter of the reference trigger when using the external
        reference mode.
        """
        self.write('RSLP {:d}'.format(value))

    @property
    def harmonic(self):
        """Getter of the detection harmonic.
        """
        return self.query('HARM?')

    @harmonic.setter
    def harmonic(self, value):
        """Setter of the detection harmonic.
        """
        self.write('HARM {:d}'.format(value))

    @property
    def sine_output_amplitude(self):
        """Getter of the amplitude of the sine output.
        """
        return self.query('SLVL?')

    @sine_output_amplitude.setter
    def sine_output_amplitude(self, value):
        """Setter of the amplitude of the sine output.
        """
        self.write('SLVL{:.3f}'.format(value))

    @property
    def input_configuration(self):
        """Getter of the configuration of the Input.
        """
        return self.query('ISRC?')

    @input_configuration.setter
    def input_configuration(self, value):
        """Setter of the configuration of the Input.
        """
        self.write('ISRC {}'.format(value))

    @property
    def input_shield(self):
        """Getter of the input shield grounding.
        """
        return self.query('IGND?')

    @input_shield.setter
    def input_shield(self, value):
        """Setter of the input shield grounding.
        """
        self.write('IGND {}'.format(value))

    @property
    def input_coupling(self):
        """Getter of the iput coupling.
        """
        return self.query('ICPL?')

    @input_coupling.setter
    def input_coupling(self, value):
        """Setter of the iput coupling.
        """
        self.write('ICPL {}'.format(value))

    @property
    def input_filter(self):
        """Getter of the input line notch filters (1x, 2x).
        """
        return self.query('ILIN?')

    @input_filter.setter
    def input_filter(self, value):
        """Setter of the input line notch filters (1x, 2x).
        """
        self.write('ILIN {}'.format(value))

    # GAIN and TIME CONSTANT COMMANDS.

    @property
    def sensitivity(self):
        """Getter of the sensitivity tuning.
        """
        return self.query('SENS?')

    @sensitivity.setter
    def sensitivity(self, value):
        """Setter of the sensitivity tuning.
        """
        self.write('SENS {}'.format(value))

    @property
    def reserve_mode(self):
        """Getter of the reserve mode tuning.
        """
        return self.query('RMOD?')

    @reserve_mode.setter
    def reserve_mode(self, value):
        """Setter of the reserve mode tuning.
        """
        self.write('RMOD {}'.format(value))

    @property
    def time_constants(self):
        """Get time constant.
        """
        return self.query('OFLT?')

    @time_constants.setter
    def time_constants(self, value):
        """Set time constant.
        """
        self.write('OFLT {}'.format(value))

    @property
    def filter_db_per_oct(self):
        """Get the low pass filter slope.
        """
        return self.query('OFSL?')

    @filter_db_per_oct.setter
    def filter_db_per_oct(self, value):
        """Set the low pass filter slope.
        """
        self.write('OFSL {:d}'.format(value))

    @property
    def sync_filter(self):
        """Get synchronous filter status.
        """
        return self.query('SYNC?')

    @sync_filter.setter
    def sync_filter(self, value):
        """Set synchronous filter status.
        """
        self.write('SYNC {}'.format(value))

    # SETUP COMMANDS

    @property
    def remote(self):
        """Lock Front panel.
        """
        self.write('OVRM?')

    @remote.setter
    def remote(self, value):
        """Lock Front panel.
        """
        self.write('OVRM {:d}'.format(value))

    @property
    def key_click_enabled(self):
        """Get key click status.
        """
        return self.query('KCLK?')

    @key_click_enabled.setter
    def key_click_enabled(self, value):
        """Set key click status.
        """
        self.write('KCLK {}'.format(value))

    @property
    def alarm_enabled(self):
        """Get alarm status.
        """
        return self.query('ALRM?')

    @alarm_enabled.setter
    def alarm_enabled(self, value):
        """Set alarm status.
        """
        self.write('ALRM {}'.format(value))

    def recall_state(self, location):
        """Recalls instrument state in specified non-volatile location.
        :param location: non-volatile storage location.
        """
        self.write('RSET {}'.format(location))

    def save_state(self, location):
        """Saves instrument state in specified non-volatile location.
        Previously stored state in location is overwritten (no error
        is generated).
        :param location: non-volatile storage location.
        """
        self.write('SSET {}'.format(location))

    # AUTO FUNCTIONS

    def auto_gain_async(self):
        """Equivalent to press the Auto Gain key in the front panel.
        Might take some time if the time constant is long.
        Does nothing if the constant is greater than 1 second.
        """
        self.write('AGAN')

    def auto_gain(self):
        """Synced auto gain command.
        """
        self.auto_gain_async()
        self.wait_bit1()

    def auto_reserve_async(self):
        """Equivalent to press the Auto Reserve key in the front panel.
        Might take some time if the time constant is long.
        """
        self.write('ARSV')

    def auto_reserve(self):
        """Synced auto reserve command
        """
        self.auto_reserve_async()
        self.wait_bit1()

    def auto_phase_async(self):
        """Equivalent to press the Auto Phase key in the front panel.
        Might take some time if the time constant is long.
        Does nothing if the phase is unstable.
        """
        self.write('ARSV')

    def auto_phase(self):
        """Synced auto phase command.
        """
        self.auto_phase_async()
        self.wait_bit1()

    def auto_offset_async(self, channel):
        """Automatically offset a given channel to zero.
        Is equivalent to press the Auto Offset Key in the front panel.
        :param channel: the name of the channel.
        :returns: None
        """
        self.write('AOFF {}'.format(channel))

    # DATA STORAGE COMMANDS

    @property
    def sample_rate(self):
        """Queries the data sample rate.
        """
        return self.query('SRAT?')

    @sample_rate.setter
    def sample_rate(self, value):
        """Sets the data sample rate.
        """
        self.write('SRAT {}'.format(value))

    @property
    def single_shot(self):
        """End of buffer mode getter:
        - if True (1 shot mode), at the end of the buffer, data storage stops
        and an audio alarm sounds.
        - if False (Loop mode), data storage continues continues at the end of
        the buffer.
        Note: If loop mode selected, make sure to pause data storage before
        reading the data to avoid confusion about which point is the most
        recent.
        """
        return self.query('SEND?')

    @single_shot.setter
    def single_shot(self, value):
        """End of buffer mode setter.
        """
        self.write('SEND {}'.format(value))

    def trigger(self):
        """Software trigger command. This command has the same effect as
        a trigger at the rear panel trigger input.
        """
        self.write('TRIG')

    @property
    def trigger_start_mode(self):
        """Getter of trigger start mode.
        """
        return self.query('TSTR?')

    @trigger_start_mode.setter
    def trigger_start_mode(self, value):
        """Setter of trigger start mode.
        """
        self.write('TSTR {}'.format(value))

    def start_data_storage(self):
        """Start or resume data storage. Command is ignored if
        storage is already in progress.
        """
        self.write('STRT')

    def pause_data_storage(self):
        """Pause data storage.  If storage is already paused
        or reset then this command is ignored.
        """
        self.write('PAUS')

    def reset_data_storage(self):
        """Reset data buffers. The command can be sent at any time, any storage
        in progress, paused or not, will be reset. The command will erase
        the data buffer.
        """
        self.write('REST')

    # OAUX See above

    def buffer_length(self):
        """Query the number of points stored in the buffer.
        :returns: the number of points stored in the buffer.
        """
        return int(self.query('SPTS?'))

    @property
    def fast_mode(self):
        """Query the data transfer mode value:
        - 0: Off
        - 1: On (DOS programs or other dedicated data collection computers)
        - 2: On (Windows Operating System Programs)
        """
        return self.query('FAST?')

    @fast_mode.setter
    def fast_mode(self, value):
        """Set the data transfer mode on and off.
        """
        self.write('FAST {:d}'.format(value))

    def start_fast_data_storage(self):
        """Start the datat storage after turning on fast data transfert.
        """
        self.write('STRD')

    # DATA TRANSFER COMMANDS

    def analog_value(self, key):
        """The command reads the value of X, Y, R, θ, CH1 or CH2 display.
        Available key = {'x', 'y', 'r', 't', '1', '2'}.
        """
        if key in 'xyrt':
            return self.query('OUTP? {}'.format(key))
        else:
            return self.query('OUTR? {}'.format(key))

    def measure(self, channels):
        """The command records the values of either 2, 3, 4, 5 or 6 parameters
        at a single instant. For example, the command is a way to query values
        of X and Y (or R and θ) which are taken at the same time.
        :param channels: list of data requested
        :returns: list of data values recorded
        """
        channel = {'x': 1, 'y': 2, 'r': 3, 't': 4,
                   '1': 5, '2': 6, '3': 7, '4': 8,
                   'f': 9, '10': 10}
        channels = ','.join(str(channel[ch]) for ch in channels)
        return [float(value) for value in
                (self.query('SNAP? {}'.format(channels))).split(',')]

    def read_buffer(self, start=0, length=None, _format='a'):
        """Queries points stored in the Channel buffer
        :param start: Index of the buffer to start.
        :param length: Number of points to read.
                       Defaults to the number of points in the buffer.
        :param _format: Transfer format
                      'a': ASCII (slow)
                      'b': IEEE Binary (fast) - NOT IMPLEMENTED
                      'c': Non-IEEE Binary (fastest) - NOT IMPLEMENTED
        """
        if _format == 'c':
            cmd = 'TRCL'
        elif _format == 'b':
            cmd = 'TRCB'
        else:
            cmd = 'TRCA'
        if not length:
            length = self.buffer_length
        self.write('{}? {},{}'.format(cmd, start, length))
        if cmd == 'TRCA':
            data = self.read()
            return [float(x) for x in data.split(',')]
        elif cmd == 'TRCB':
            return self.read()
        else:
            return self.read()

    @property
    def display(self, channel):
        """Query the display source.
        :param channel: 1 or 2 i.e. CH1 or CH2 (int)
        """
        return [int(var) for var in
                self.query('DDEF? {}'.format(channel)).split(',')]

    @display.setter
    def display(self, channel, display, ratio):
        """Set the display source.
        :param channel: 1 or 2 i.e. CH1 or CH2 (int)
        :param display: 0, 1, 2, 3, 4 i.e. X, R, Xnoise,
                        AUxIn1 or AuxIn2 (int))
        :param ratio: 0, 1, 2 i.e. none, AuxIn1, AuxIn2 (int)
        """
        self.write('DDEF {},{},{}'.format(channel, display, ratio))

    @property
    def front_output(self, channel):
        """Query the output source.
        :param channel: 1 or 2 i.e. CH1 or CH2 (int)
        """
        return self.query('FPOP? {}'.format(channel))

    @front_output.setter
    def front_output(self, channel, output):
        """Set the output source.
        :param channel: 1 or 2 i.e. CH1 or CH2 (int)
        :param output: X, Y (output=1) or Display (output=0) (int)
        """
        self.write('FPOP {},{}'.format(channel, output))
